@@ -36,6 +36,12 @@
 
 #include <IOKit/usb/IOUSBRootHubDevice.h>
 #include <IOKit/usb/IOUSBLog.h>
+#include <IOKit/acpi/IOACPIPlatformDevice.h>
+#include <libkern/libkern.h>
+
+#ifndef kACPIDevicePathKey
+#define kACPIDevicePathKey			"acpi-path"
+#endif
 
 #include "AppleUSBEHCI.h"
 
@@ -51,6 +57,8 @@ enum {
 
 #include "AppleUSBEHCI.h"
 
+#define super IOUSBControllerV2
+
 // USB bus has two power states, off and on
 #define number_of_power_states 2
 
@@ -66,6 +74,8 @@ static IOPMPowerState ourPowerStatesKL[number_of_power_states] = {
   {1, IOPMDeviceUsable, IOPMPowerOn, IOPMPowerOn | IOPMClockNormal, 0,0,0,0,0,0,0,0}
 };
 
+extern UInt32 getPortSCForWriting(EHCIRegistersPtr _pEHCIRegisters, short port);
+
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 // initForPM
@@ -152,6 +162,11 @@ AppleUSBEHCI::initForPM (IOPCIDevice *provider)
 	// this is for restarts and shut downs
 	if (_errataBits & kErrataICH6PowerSequencing)
 		_powerDownNotifier = registerPrioritySleepWakeInterest(PowerDownHandler, this, 0);
+		
+	// if we have an ExpressCard attached (non-zero port), then we will need to disable port resume 
+	// for that port (some cards disconnect when the ExpressCard power goes away and we would like to ignore these extra detach events.
+	_ExpressCardPort = ExpressCardPort(provider);	
+	_badExpressCardAttached = false;
 }
 
 
@@ -451,7 +466,8 @@ IOReturn AppleUSBEHCI::callPlatformFunction(const OSSymbol *functionName,
 											void *param1, void *param2,
 											void *param3, void *param4)
 {  
-    IOLog("EHCIUIM -- callPlatformFunction unimplimented \n");
+    USBLog(3, "%s[%p]::callPlatformFunction(%s)", getName(), this, functionName->getCStringNoCopy());
+
     if (functionName == usb_remote_wakeup)
     {
 		bool	*wake;
@@ -469,7 +485,7 @@ IOReturn AppleUSBEHCI::callPlatformFunction(const OSSymbol *functionName,
     	return kIOReturnSuccess;
     }
 	
-    return kIOReturnBadArgument;
+    return super::callPlatformFunction(functionName, waitForFunction, param1, param2, param3, param4);
 }
 
 
@@ -477,7 +493,7 @@ void
 AppleUSBEHCI::ResumeUSBBus()
 {
     UInt8	numPorts;
-    int 	i;
+	int		i;
     bool	enabledports = false;
 
     // restore volatile registers saved in SuspendUSBBus()
@@ -539,13 +555,19 @@ AppleUSBEHCI::ResumeUSBBus()
     for (i=0; i < numPorts; i++)
     {
 		UInt32 portStat;
-		portStat = USBToHostLong(_pEHCIRegisters->PortSC[i]);
+		portStat = getPortSCForWriting(_pEHCIRegisters, i+1);
 		if (portStat & kEHCIPortSC_Owner)
 		{
 			USBLog(4, "AppleUSBEHCI[%p]::ResumeUSBBus - port %d owned by OHCI",  this, i+1);
 		}
 		else if (portStat & kEHCIPortSC_Enabled)
 		{
+			// is this an ExpressCard port that we disabled resume enable?  - if so, put it back!
+			if (_badExpressCardAttached && ((int)_ExpressCardPort == (i+1))){
+				portStat |= (kEHCIPortSC_WKCNNT_E|kEHCIPortSC_WKDSCNNT_E);
+				_pEHCIRegisters->PortSC[i] = USBToHostLong(portStat);
+				IOSync();
+			}
 			// If the port was NOT suspended prior to the suspend OR it is the cause of the resume, then
 			// resume the port
 			if ( !(_savedSuspendedPortBitmap & (1<<i)) || (portStat & kEHCIPortSC_Resume) )
@@ -591,7 +613,7 @@ AppleUSBEHCI::ResumeUSBBus()
 		for (i=0; i < numPorts; i++)
 		{
 			UInt32 portStat;
-			portStat = USBToHostLong(_pEHCIRegisters->PortSC[i]);
+			portStat = getPortSCForWriting(_pEHCIRegisters, i+1);
 			if (portStat & kEHCIPortSC_Owner)
 			{
 				USBLog(7, "AppleUSBEHCI[%p]::ResumeUSBBus - port %d owned by OHCI",  this, i+1);
@@ -721,13 +743,20 @@ AppleUSBEHCI::SuspendUSBBus()
     for (i=0; i < numPorts; i++)
     {
 		UInt32 portStat;
-		portStat = USBToHostLong(_pEHCIRegisters->PortSC[i]);
+		portStat = getPortSCForWriting(_pEHCIRegisters, i+1);
 		if (portStat & kEHCIPortSC_Owner)
 		{
 			USBLog(4, "AppleUSBEHCI[%p]::SuspendUSBBus - port %d owned by OHCI",  this, i+1);
 		}
 		else if (portStat & kEHCIPortSC_Enabled)
 		{
+			// is this an ExpressCard port that needs to turn off resume enable?
+			if (_badExpressCardAttached && ((int)_ExpressCardPort == (i+1))){
+				portStat &= ~(kEHCIPortSC_WKCNNT_E|kEHCIPortSC_WKDSCNNT_E);
+				_pEHCIRegisters->PortSC[i] = USBToHostLong(portStat);
+				IOSync();
+			}
+			
 			if (portStat & kEHCIPortSC_Suspend)
 			{
 				_savedSuspendedPortBitmap |= (1<<i);
@@ -847,3 +876,106 @@ AppleUSBEHCI::PowerDownHandler(void *target, void *refCon, UInt32 messageType, I
     return kIOReturnSuccess;
 }
 
+static IOACPIPlatformDevice * CopyACPIDevice( IORegistryEntry * device )
+{
+	IOACPIPlatformDevice *  acpiDevice = 0;
+	OSString *				acpiPath;
+
+	if (device)
+	{
+		acpiPath = (OSString *) device->copyProperty(kACPIDevicePathKey);
+		if (acpiPath && !OSDynamicCast(OSString, acpiPath))
+		{
+			acpiPath->release();
+			acpiPath = 0;
+		}
+
+		if (acpiPath)
+		{
+			IORegistryEntry * entry;
+
+			entry = IORegistryEntry::fromPath(acpiPath->getCStringNoCopy());
+			acpiPath->release();
+
+			if (entry && entry->metaCast("IOACPIPlatformDevice"))
+				acpiDevice = (IOACPIPlatformDevice *) entry;
+			else if (entry)
+				entry->release();
+		}
+	}
+
+	return (acpiDevice);
+}
+
+static bool HasExpressCardUSB( IORegistryEntry * acpiDevice, UInt32 * portnum )
+{
+	const IORegistryPlane *	acpiPlane;
+	bool					match = false;
+	IORegistryIterator *	iter;
+	IORegistryEntry *		entry;
+
+	do {
+		acpiPlane = acpiDevice->getPlane( "IOACPIPlane" );
+		if (!acpiPlane)
+			break;
+
+		// acpiDevice is the USB controller in ACPI plane.
+		// Recursively iterate over children of acpiDevice.
+
+		iter = IORegistryIterator::iterateOver(
+				/* start */	acpiDevice,
+				/* plane */	acpiPlane,
+				/* options */ kIORegistryIterateRecursively);
+
+		if (iter)
+		{
+			while (!match && (entry = iter->getNextObject()))
+			{
+				// USB port must be a leaf node (no child), and
+				// must be an IOACPIPlatformDevice.
+
+				if ((entry->getChildEntry(acpiPlane) == 0) &&
+					entry->metaCast("IOACPIPlatformDevice"))
+				{
+					IOACPIPlatformDevice * port;
+					port = (IOACPIPlatformDevice *) entry;
+
+					// Express card port? Is port ejectable?
+
+					if (port->validateObject( "_EJD" ) == kIOReturnSuccess)
+					{
+						// Determining the USB port number.
+						if (portnum)
+							*portnum = strtoul(port->getLocation(), NULL, 10);
+						match = true;
+					}
+				}
+			}
+
+			iter->release();
+		}
+	}
+	while (false);
+	
+	return match;
+}
+
+// Checks for ExpressCard connected to this controller, and returns the port number (1 based)
+// Will return 0 if no ExpressCard is connected to this controller.
+//
+UInt32 AppleUSBEHCI::ExpressCardPort( IOService * provider )
+{
+	IOACPIPlatformDevice *	acpiDevice;
+	UInt32					portNum = 0;
+	bool					isPCIeUSB;
+	
+	acpiDevice = CopyACPIDevice( provider );
+	if (acpiDevice)
+	{
+		isPCIeUSB = HasExpressCardUSB( acpiDevice, &portNum );	
+		acpiDevice->release();
+	}
+	return(portNum);
+}
+
+
