@@ -2,6 +2,8 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
+ * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -26,24 +28,25 @@
 #include "AppleUSBEHCI.h"
 #include "USBEHCI.h"
 
-class AppleEHCIitdMemoryBlock : public IOBufferMemoryDescriptor
+class AppleEHCIitdMemoryBlock : public OSObject
 {
-    OSDeclareDefaultStructors(AppleEHCIitdMemoryBlock);
+	OSDeclareDefaultStructors(AppleEHCIitdMemoryBlock)
     
 #define ITDsPerBlock	(kEHCIPageSize / sizeof(EHCIIsochTransferDescriptorShared))
 
 private:
-    IOPhysicalAddress				_sharedPhysical;
+    IOPhysicalAddress						_sharedPhysical;
     EHCIIsochTransferDescriptorSharedPtr	_sharedLogical;
-    AppleEHCIitdMemoryBlock			*_nextBlock;
+    AppleEHCIitdMemoryBlock					*_nextBlock;
+	IOBufferMemoryDescriptor				*_buffer;
     
 public:
 
-    static AppleEHCIitdMemoryBlock 		*NewMemoryBlock(void);
-    void					SetNextBlock(AppleEHCIitdMemoryBlock *next);
-    AppleEHCIitdMemoryBlock			*GetNextBlock(void);
-    UInt32					NumTDs(void);
-    IOPhysicalAddress				GetPhysicalPtr(UInt32 index);
+    static AppleEHCIitdMemoryBlock			*NewMemoryBlock(void);
+    void									SetNextBlock(AppleEHCIitdMemoryBlock *next);
+    AppleEHCIitdMemoryBlock					*GetNextBlock(void);
+    UInt32									NumTDs(void);
+    IOPhysicalAddress						GetPhysicalPtr(UInt32 index);
     EHCIIsochTransferDescriptorSharedPtr	GetLogicalPtr(UInt32 index);
     
 };
