@@ -38,7 +38,7 @@ enum
 };
 
 
-class AppleUSBOHCIedMemoryBlock : public IOBufferMemoryDescriptor
+class AppleUSBOHCIedMemoryBlock : public OSObject
 {
     OSDeclareDefaultStructors(AppleUSBOHCIedMemoryBlock);
     
@@ -49,6 +49,7 @@ private:
     OHCIEndpointDescriptorSharedPtr		_sharedLogical;
     AppleUSBOHCIedMemoryBlock			*_nextBlock;
     AppleOHCIEndpointDescriptor			_eds[EDsPerBlock];	// the non shared data
+	IOBufferMemoryDescriptor			*_buffer;
     
 public:
 
@@ -64,7 +65,7 @@ public:
 
 
 
-class AppleUSBOHCIgtdMemoryBlock : public IOBufferMemoryDescriptor
+class AppleUSBOHCIgtdMemoryBlock : public OSObject
 {
     OSDeclareDefaultStructors(AppleUSBOHCIgtdMemoryBlock);
     
@@ -75,6 +76,7 @@ private:
     OHCIGeneralTransferDescriptorSharedPtr		_sharedLogical;
     AppleUSBOHCIgtdMemoryBlock					*_nextBlock;
     AppleOHCIGeneralTransferDescriptor			_gtds[GTDsPerBlock];	// the non shared data
+	IOBufferMemoryDescriptor					*_buffer;
     
 public:
 
@@ -91,7 +93,7 @@ public:
 
 
 
-class AppleUSBOHCIitdMemoryBlock : public IOBufferMemoryDescriptor
+class AppleUSBOHCIitdMemoryBlock : public OSObject
 {
     OSDeclareDefaultStructors(AppleUSBOHCIitdMemoryBlock);
     
@@ -102,6 +104,7 @@ private:
     OHCIIsochTransferDescriptorSharedPtr			_sharedLogical;
     AppleUSBOHCIitdMemoryBlock						*_nextBlock;
     AppleOHCIIsochTransferDescriptor				_itds[ITDsPerBlock];	// the non shared data
+	IOBufferMemoryDescriptor						*_buffer;
     
 public:
 
