@@ -33,10 +33,13 @@ extern "C" {
 #include <IOKit/IOPlatformExpert.h>
 #include <IOKit/pccard/IOPCCard.h>
 #include <IOKit/platform/ApplePlatformExpert.h>
+#include <IOKit/IOKitKeys.h>
+#include <IOKit/IOBufferMemoryDescriptor.h>
+#include <IOKit/IODMACommand.h>
+
 #include <IOKit/usb/USB.h>
 #include <IOKit/usb/IOUSBLog.h>
-
-#include <IOKit/IOBufferMemoryDescriptor.h>
+#include <IOKit/usb/IOUSBRootHubDevice.h>
 
 #include <libkern/OSAtomic.h>
 
@@ -56,42 +59,6 @@ extern "C" {
  */
 
 OSDefineMetaClassAndStructors(AppleUSBUHCI, IOUSBControllerV2)
-OSDefineMetaClassAndStructors(UHCIMemoryBuffer, IOBufferMemoryDescriptor)
-
-
-UHCIMemoryBuffer *
-UHCIMemoryBuffer::newBuffer(bool dmaable)
-{
-    UHCIMemoryBuffer *bp;
-    
-    bp = new UHCIMemoryBuffer;
-    if (bp == NULL) 
-	{
-        return NULL;
-    }
-    
-    if (!bp->initWithOptions(kIOMemoryUnshared | kIODirectionInOut, PAGE_SIZE, PAGE_SIZE))
-	{
-        bp->release();
-        return NULL;
-    }
-    if (dmaable) 
-	{
-        bp->prepare();
-    }
-    
-    return bp;
-}
-
-
-
-void
-UHCIMemoryBuffer::free()
-{
-    complete();
-    IOBufferMemoryDescriptor::free();
-}
-
 
 
 // ========================================================================
@@ -216,7 +183,6 @@ AppleUSBUHCI::finalize(IOOptionBits options)
 
 
 
-
 void
 AppleUSBUHCI::EnableUSBInterrupt(bool enableInterrupt)
 {
@@ -241,12 +207,9 @@ IOReturn
 AppleUSBUHCI::HardwareInit(void)
 {
     IOReturn									status;
-    UInt32 *									frames;
-    IOPhysicalAddress							pPhysical;
     int											i, j, frame_period;
     AppleUHCITransferDescriptor					*pTD;
     AppleUHCIQueueHead							*lastQH, *bulkQH, *fsQH, *lsQH, *pQH;
-    UHCIMemoryBuffer							*bp;
     
     ioWrite16(kUHCI_INTR, 0);					// Disable interrupts
     
@@ -257,24 +220,17 @@ AppleUSBUHCI::HardwareInit(void)
         return status;
     }
     
-    // Set up frame array
-    bp = UHCIMemoryBuffer::newBuffer();
-    if (bp == NULL) 
+	status = InitializeBufferMemory();
+    if (status != kIOReturnSuccess) 
 	{
-        return kIOReturnNoMemory;
+		USBError(1, "AppleUSBUHCI[%p]::HardwareInit - InitializeBufferMemory failed with status(%p)", this, (void*)status);
+        return status;
     }
-    queue_enter(&_allocatedBuffers, bp, UHCIMemoryBuffer *, _chain);
-    
-    frames = (UInt32 *)bp->getBytesNoCopy();
-    pPhysical = bp->getPhysicalAddress();
-	USBLog(7, "AppleUSBUHCI[%p]::HardwareInit - frame list pPhysical[%p] frames[%p]", this, (void*)pPhysical, frames);
-    _framesPaddr = pPhysical;
-	_frameList = frames;								// pointer to the list of physical addresses
-    
+
     // Set frame number and physical frame address
     ioWrite16(kUHCI_FRNUM, 0);
-    ioWrite32(kUHCI_FRBASEADDR, pPhysical);
-    USBLog(7, "AppleUSBUHCI[%p]::HardwareInit - Setting physical frame address to %p", this, (void*)pPhysical);
+    ioWrite32(kUHCI_FRBASEADDR, _framesPaddr);
+    USBLog(7, "AppleUSBUHCI[%p]::HardwareInit - Setting physical frame address to %p", this, (void*)_framesPaddr);
     
     //============= Set up queue heads =======================//
     
@@ -348,7 +304,7 @@ AppleUSBUHCI::HardwareInit(void)
 		frame_period = (1 << i);
 		for (j=frame_period-1; j < kUHCI_NVFRAMES; j += frame_period)
 		{
-			frames[j] = HostToUSBLong(pQH->GetPhysicalAddrWithType());
+			_frameList[j] = HostToUSBLong(pQH->GetPhysicalAddrWithType());
 			_logicalFrameList[j] = pQH;
 		}
         lastQH = pQH;
@@ -418,9 +374,7 @@ AppleUSBUHCI::UIMInitialize(IOService * provider)
 		{
             return kIOReturnNoMemory;
         }
-        
-        queue_init(&_allocatedBuffers);
-		
+
         _isocBandwidth = kUSBMaxFSIsocEndpointReqCount;
         _uhciBusState = kUHCIBusStateRunning;
 		
@@ -532,22 +486,24 @@ AppleUSBUHCI::UIMFinalize()
     }
 	
     USBLog(3, "AppleUSBUHCI[%p]::UIMFinalize freeing memory", this);
-    
-    // Free allocated TD, QH, transaction and frame memory.
 	
-    while (!queue_empty(&_allocatedBuffers)) 
+	FreeBufferMemory();
+	
+	/*// free the frame list IODMACommand and IOBufferMemoryDescriptor
+	if (_frameListBuffer)
 	{
-        UHCIMemoryBuffer *bp;
-        
-        queue_remove_first(&_allocatedBuffers, bp, UHCIMemoryBuffer *, _chain);
-        bp->release();
-    }
-    
+		_frameListBuffer->complete();
+		_frameListBuffer->release();
+		_frameListBuffer = NULL;
+	}
+	*/
     // TODO: free the transfer descriptor memory blocks
     // TODO: free the queue head memory blocks
 	
     if (_rhTimer)
     {
+		_rhTimer->cancelTimeout();
+		
         if ( _workLoop )
             _workLoop->removeEventSource(_rhTimer);
         
@@ -665,6 +621,22 @@ AppleUSBUHCI::UIMFinalizeForPowerDown()
 IOReturn
 AppleUSBUHCI::message( UInt32 type, IOService * provider,  void * argument )
 {
+	if (type == kIOUSBMessageExpressCardCantWake)
+	{
+		IOService *					nub = (IOService*)argument;
+		const IORegistryPlane *		usbPlane = getPlane(kIOUSBPlane);
+		IOUSBRootHubDevice *		parentHub = OSDynamicCast(IOUSBRootHubDevice, nub->getParentEntry(usbPlane));
+
+		nub->retain();
+		USBLog(1, "AppleUSBUHCI[%p]::message - got kIOUSBMessageExpressCardCantWake from driver %s[%p] argument is %s[%p]", this, provider->getName(), provider, nub->getName(), nub);
+		if (parentHub == _rootHubDevice)
+		{
+			USBLog(1, "AppleUSBUHCI[%p]::message - device is attached to my root hub (port %d)!!", this, (int)_ExpressCardPort);
+			_badExpressCardAttached = true;
+		}
+		nub->release();
+		return kIOReturnSuccess;
+	}
     return super::message( type, provider, argument );
 }
 
@@ -1230,6 +1202,31 @@ AppleUSBUHCI::scavengeAnIsochTD(AppleUHCIIsochTransferDescriptor *pTD)
 
 
 
+void
+AppleUSBUHCI::PutTDonDoneQueue(IOUSBControllerIsochEndpoint* pED, IOUSBControllerIsochListElement *pTD, bool checkDeferred)
+{
+	AppleUHCIIsochTransferDescriptor	*pUHCITD = OSDynamicCast(AppleUHCIIsochTransferDescriptor, pTD);
+	if (pUHCITD && pUHCITD->alignBuffer)
+	{
+		if (pED->direction == kUSBOut)
+		{
+			USBLog(7, "AppleUSBUHCI[%p]::PutTDonDoneQueue - found alignment buffer on Isoch OUT (%p) - freeing", this, pUHCITD->alignBuffer);
+			ReleaseIsochAlignmentBuffer(pUHCITD->alignBuffer);
+		}
+		else if (pUHCITD->alignBuffer->dmaCommand)
+		{
+			// put these in the dma command to be copied when the dmaCommand is completed
+			USBLog(7, "AppleUSBUHCI[%p]::PutTDonDoneQueue - found alignment buffer on Isoch IN (%p) - storing in dmacommand (%p)", this, pUHCITD->alignBuffer, pUHCITD->alignBuffer->dmaCommand);
+			queue_enter(&pUHCITD->alignBuffer->dmaCommand->_alignment_buffers, pUHCITD->alignBuffer, UHCIAlignmentBuffer *, chain);
+		}
+		pUHCITD->alignBuffer = NULL;
+	}
+	
+	IOUSBControllerV2::PutTDonDoneQueue(pED, pTD, checkDeferred);
+}
+
+
+
 IOReturn						
 AppleUSBUHCI::scavengeQueueHeads(IOUSBControllerListElement *pLE)
 {
@@ -1243,25 +1240,32 @@ AppleUSBUHCI::scavengeQueueHeads(IOUSBControllerListElement *pLE)
     while( (pLE != NULL) && (leCount++ < 150000) )
     {
 		pQH = OSDynamicCast(AppleUHCIQueueHead, pLE);
+		tdCount = 0;
+		
 		if(pQH && (pQH->type != kQHTypeDummy) && (!pQH->stalled))
 		{
 			bool	foundInactive = false;
 			
 			qTD = qHead = pQH->firstTD;
 			qEnd = pQH->lastTD;
-			if (((qTD == NULL) || (qEnd == NULL)) && (qTD != qEnd))
+			if (((qHead == NULL) || (qEnd == NULL)) && (qHead != qEnd))
 			{
-				USBError(1, "The UHCI driver found a device queue with invalid head (%p) or tail (%p)", qTD, qEnd);
+				USBError(1, "The UHCI driver found a device queue with invalid head (%p) or tail (%p)", qHead, qEnd);
 			}
 			TDisHalted = false;
 			shortTransfer = false;
+			
+			// reset
+			tdCount = 0;
+			qTD = pQH->firstTD;
+			
 			if (qTD && (qTD != qEnd))
 			{
 				USBLog(7, "AppleUSBUHCI[%p]::scavengeQueueHeads - looking at pQH[%p]=========================================", this, pQH);
 				logging = true;
 			}
 				
-			while( qTD && (qTD != qEnd) && (tdCount++ < 150000) )
+			while(qTD && (qTD != qEnd) && (tdCount++ < 150000) )
 			{	
 				// This end point has transactions
 				ctrlStatus = USBToHostLong(qTD->GetSharedLogical()->ctrlStatus);
@@ -1304,10 +1308,39 @@ AppleUSBUHCI::scavengeQueueHeads(IOUSBControllerListElement *pLE)
 						pQH->stalled = true;
 					}
 				}
-				if (qTD->buffer && (qTD->direction == kUSBIn) && actLength)
+				if (qTD->alignBuffer)
 				{
-					USBLog(1, "AppleUSBUHCI[%p]::scavengeQueueHeads - writing back from alignment buffer", this);
-					qTD->buffer->userBuffer->writeBytes(qTD->buffer->userOffset, (void*)qTD->buffer->vaddr, actLength);
+					IOUSBCommand	*command = qTD->command;
+					
+					if ((qTD->direction == kUSBOut) || !actLength)
+					{
+						USBLog(1, "AppleUSBUHCI[%p]::scavengeQueueHeads - releasing CBI buffer (%p) - direction (%s) - actLen (%d)", this, qTD->alignBuffer, qTD->direction == kUSBOut ? "OUT" : "IN", actLength);
+						ReleaseCBIAlignmentBuffer(qTD->alignBuffer);
+						qTD->alignBuffer = NULL;
+					}
+					else
+					{
+						// for IN transactions, we store them in the DMA Command to be copied after the DMACommand is released
+						if (!command)
+						{
+							USBError(1, "AppleUSBUHCI[%p]::scavengeQueueHeads - ERROR - missing usbcommand!!", this);
+						}
+						else
+						{
+							AppleUSBUHCIDMACommand	*dmaCommand = OSDynamicCast(AppleUSBUHCIDMACommand, command->GetDMACommand());
+							if (dmaCommand && (dmaCommand->getMemoryDescriptor()))
+							{
+								USBLog(1, "AppleUSBUHCI[%p]::scavengeQueueHeads - IN transaction - storing UHCIAlignmentBuffer (%p) into dmaCommand (%p) to be copied later - actLegth (%d)", this, qTD->alignBuffer, dmaCommand, actLength);
+								qTD->alignBuffer->actCount = actLength;
+								queue_enter(&dmaCommand->_alignment_buffers, qTD->alignBuffer, UHCIAlignmentBuffer *, chain);
+								qTD->alignBuffer = NULL;
+							}
+							else
+							{
+								USBError(1, "AppleUSBUHCI[%p]::scavengeQueueHeads - ERROR - TD (%p) missing or empty dmaCommand (%p) or (%p)", this, qTD, dmaCommand, command->GetDMACommand());
+							}
+						}
+					}
 				}
 				if (qTD->lastTDofTransaction)
 				{
@@ -1365,12 +1398,12 @@ AppleUSBUHCI::scavengeQueueHeads(IOUSBControllerListElement *pLE)
 					}
 					// we are going to return the TDs between the curent firstTD and the new qTD, so change the firstTD
 					pQH->firstTD = qTD;
-					
-                    // Reset our loop variables
-                    //
-                    TDisHalted = false;
-                    shortTransfer = false;
-                } 
+
+					// Reset our loop variables
+					//
+					TDisHalted = false;
+					shortTransfer = false;
+				} 
 				else
 				{
 					USBLog(7, "AppleUSBUHCI[%p]::scavengeQueueHeads - looking past TD (%p) to TD (%p)", this, qTD, qTD->_logicalNext); 
@@ -1450,7 +1483,7 @@ AppleUSBUHCI::UHCIUIMDoDoneQueueProcessing(AppleUHCITransferDescriptor *pHCDoneT
 				UInt32 value = USBToHostLong(pHCDoneTD->GetSharedLogical()->ctrlStatus);
 				if ( value & kUHCI_TD_BABBLE )
 				{
-					USBLog(4, "AppleUSBUHCI[%p]::UHCIUIMDoDoneQueueProcessing - TD (%p) had the BABBLE bit on (0x%x), calling UIMRootHubStatusChange directly()", this, pHCDoneTD, value);
+					USBLog(4, "AppleUSBUHCI[%p]::UHCIUIMDoDoneQueueProcessing - TD (%p) had the BABBLE bit on (0x%x), calling UIMRootHubStatusChange directly()", this, pHCDoneTD, (unsigned int)value);
 					UIMRootHubStatusChange();
 				}
 				
@@ -1485,12 +1518,12 @@ AppleUSBUHCI::UHCIUIMDoDoneQueueProcessing(AppleUHCITransferDescriptor *pHCDoneT
 						else
 						{
 							_controlBulkTransactionsOut--;
-							USBLog(6, "AppleUSBUHCI[%p]::UHCIUIMDoDoneQueueProcessing - _controlBulkTransactionsOut(%p) pHCDoneTD(%p)", this, (void*)_controlBulkTransactionsOut, pHCDoneTD);
+							USBLog(7, "AppleUSBUHCI[%p]::UHCIUIMDoDoneQueueProcessing - _controlBulkTransactionsOut(%p) pHCDoneTD(%p)", this, (void*)_controlBulkTransactionsOut, pHCDoneTD);
 							if (!_controlBulkTransactionsOut)
 							{
 								UInt32 link;
 								link = _lastQH->GetPhysicalLink();
-								USBLog(6, "AppleUSBUHCI[%p]::UHCIUIMDoDoneQueueProcessing - no more _controlBulkTransactionsOut - terminating list (%p to %p)", this, (void*)link, (void*)(link | kUHCI_QH_T));
+								USBLog(7, "AppleUSBUHCI[%p]::UHCIUIMDoDoneQueueProcessing - no more _controlBulkTransactionsOut - terminating list (%p to %p)", this, (void*)link, (void*)(link | kUHCI_QH_T));
 								_lastQH->SetPhysicalLink(link | kUHCI_QH_T);
 							}
 						}
@@ -1567,7 +1600,7 @@ AppleUSBUHCI::AllocateTD(AppleUHCIQueueHead *pQH)
 		if (!_pFreeTD)
 			_pLastFreeTD = NULL;
 		freeTD->_logicalNext = NULL;
-		freeTD->buffer = NULL;										// no alignment buffer yet
+		freeTD->alignBuffer = NULL;										// no alignment buffer yet
 		freeTD->lastFrame = 0;
 		freeTD->lastRemaining = 0;
 		freeTD->command = NULL;
@@ -1592,14 +1625,6 @@ AppleUSBUHCI::DeallocateTD(AppleUHCITransferDescriptor *pTD)
 	pTD->GetSharedLogical()->ctrlStatus = 0;
     pTD->_logicalNext = NULL;
 	
-	if (pTD->buffer)
-	{
-		USBLog(3, "AppleUSBUHCI[%p]::DeallocateTD - have alignment buffer %p and pQH %p", this, pTD->buffer, pQH);
-		if (pQH)
-			pQH->ReleaseAlignmentBuffer(pTD->buffer);
-		pTD->buffer = NULL;
-	}
-	
     if (_pLastFreeTD)
     {
         _pLastFreeTD->_logicalNext = pTD;
@@ -1665,7 +1690,7 @@ AppleUSBUHCI::AllocateITD(void)
 		if (!_pFreeITD)
 			_pLastFreeITD = NULL;
 		freeITD->_logicalNext = NULL;
-		freeITD->buffer = NULL;											// no alignment buffer
+		freeITD->alignBuffer = NULL;											// no alignment buffer
 		// zero out the shared data
 		freeITD->GetSharedLogical()->ctrlStatus = 0;
 		freeITD->SetPhysicalLink(0);
@@ -1681,17 +1706,15 @@ IOReturn
 AppleUSBUHCI::DeallocateITD(AppleUHCIIsochTransferDescriptor *pITD)
 {
     UInt32					physical;
-	AppleUHCIIsochEndpoint	*pEP = OSDynamicCast(AppleUHCIIsochEndpoint, pITD->_pEndpoint);
 	
 	pITD->GetSharedLogical()->ctrlStatus = 0;
     pITD->_logicalNext = NULL;
 	
-	if (pITD->buffer)
+	if (pITD->alignBuffer)
 	{
-		USBLog(6, "AppleUSBUHCI[%p]::DeallocateITD - have alignment buffer %p and pEP %p", this, pITD->buffer, pEP);
-		if (pEP)
-			pEP->ReleaseAlignmentBuffer(pITD->buffer);
-		pITD->buffer = NULL;
+		USBError(1, "AppleUSBUHCI[%p]::DeallocateITD - UNEXPECTED alignment buffer %p", this, pITD->alignBuffer);
+		ReleaseIsochAlignmentBuffer(pITD->alignBuffer);
+		pITD->alignBuffer = NULL;
 	}
 	
     if (_pLastFreeITD)
@@ -1759,7 +1782,6 @@ AppleUSBUHCI::AllocateQH(UInt16 functionNumber, UInt16 endpointNumber, UInt8 dir
 		if (!_pFreeQH)
 			_pLastFreeQH = NULL;
 		freeQH->_logicalNext = NULL;
-		freeQH->buffersInUse = 0;
 		freeQH->functionNumber = functionNumber;
 		freeQH->endpointNumber = endpointNumber;
 		freeQH->direction = direction;
@@ -1767,8 +1789,6 @@ AppleUSBUHCI::AllocateQH(UInt16 functionNumber, UInt16 endpointNumber, UInt8 dir
 		freeQH->maxPacketSize = maxPacketSize;
 		freeQH->type = type;
         freeQH->stalled = false;
-		queue_init(&freeQH->allocatedBuffers);
-		queue_init(&freeQH->freeBuffers);
 	}
     return freeQH;
 }
@@ -1802,9 +1822,9 @@ AppleUSBUHCI::DeallocateQH(AppleUHCIQueueHead *pQH)
 IOUSBControllerIsochEndpoint*			
 AppleUSBUHCI::AllocateIsochEP()
 {
-	AppleUHCIIsochEndpoint		*pEP;
+	IOUSBControllerIsochEndpoint		*pEP;
 	
-	pEP = new AppleUHCIIsochEndpoint;
+	pEP = new IOUSBControllerIsochEndpoint;
 	if (pEP)
 	{
 		if (!pEP->init())
@@ -1818,6 +1838,420 @@ AppleUSBUHCI::AllocateIsochEP()
 
 
 
+IOReturn
+AppleUSBUHCI::GetLowLatencyOptionsAndPhysicalMask(IOOptionBits *pOptionBits, mach_vm_address_t *pPhysicalMask)
+{
+	super::GetLowLatencyOptionsAndPhysicalMask(pOptionBits, pPhysicalMask);				// get the defaults
+	*pOptionBits = kIOMemoryPhysicallyContiguous;										// make sure we are physically contiguous
+	return kIOReturnSuccess;
+}
+
+
+
+IOReturn
+AppleUSBUHCI::InitializeBufferMemory()
+{
+	IOReturn									status;
+	char *										logicalBytes;
+	UInt64										offset = 0;
+	IODMACommand::Segment32						segments;
+	UInt32										numSegments = 1;
+    IOPhysicalAddress							pPhysical= 0;
+	IODMACommand *								dmaCommand = NULL;
+	UHCIAlignmentBuffer							*alignBuf;
+	bool										alignBufferPrepared = false;
+	bool										isochBufferPrepared = false;
+	bool										frameBufferPrepared = false;
+	int											i,j;
+	
+	// make sure that things are initialized to NULL
+	_cbiAlignBuffer = NULL;
+	_isochAlignBuffer = NULL;
+	queue_init(&_cbiAlignmentBuffers);
+	queue_init(&_isochAlignmentBuffers);
+
+	// Use IODMACommand to get the physical address
+	dmaCommand = IODMACommand::withSpecification(kIODMACommandOutputHost32, 32, PAGE_SIZE, (IODMACommand::MappingOptions)(IODMACommand::kMapped | IODMACommand::kIterateOnly));
+	if (!dmaCommand)
+	{
+		USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not create IODMACommand", this);
+		return kIOReturnInternalError;
+	}
+	USBLog(6, "AppleUSBUHCI[%p]::InitializeBufferMemory - got IODMACommand %p", this, dmaCommand);
+	
+	// the old do while false loop to prevent goto statements
+	do
+	{
+		// Set up frame array
+		_frameListBuffer = IOBufferMemoryDescriptor::inTaskWithPhysicalMask(kernel_task, kIOMemoryUnshared | kIODirectionInOut, PAGE_SIZE, kUHCIStructureAllocationPhysicalMask);
+		if (_frameListBuffer == NULL) 
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not get frame list buffer", this);
+			status = kIOReturnNoMemory;
+			break;
+		}
+		
+		status = _frameListBuffer->prepare();
+		if (status)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - _frameListBuffer->prepare failed with status(%p)", this, (void*)status);
+			break;
+		}
+		
+		frameBufferPrepared = true;
+		
+		status = dmaCommand->setMemoryDescriptor(_frameListBuffer);
+		if (status)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - setMemoryDescriptor returned err (%p)", this, (void*)status);
+			break;
+		}
+		
+		offset = 0;
+		segments.fIOVMAddr = 0;
+		segments.fLength = 0;
+		numSegments = 1;
+		
+		status = dmaCommand->gen32IOVMSegments(&offset, &segments, &numSegments);
+		if (status || (numSegments != 1) || (segments.fLength != PAGE_SIZE))
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not generate segments err (%p) numSegments (%d) fLength (%d)", this, (void*)status, (int)numSegments, (int)segments.fLength);
+			dmaCommand->clearMemoryDescriptor();
+			status = status ? status : kIOReturnInternalError;
+			break;
+		}
+		
+		_frameList = (UInt32 *)_frameListBuffer->getBytesNoCopy();
+		pPhysical = segments.fIOVMAddr;
+		
+		USBLog(7, "AppleUSBUHCI[%p]::HardwareInit - frame list pPhysical[%p] frames[%p]", this, (void*)pPhysical, _frameList);
+		_framesPaddr = pPhysical;
+		dmaCommand->clearMemoryDescriptor();
+		
+		// set up some alignment buffers for control/bulk/interrupt
+		_cbiAlignBuffer = IOBufferMemoryDescriptor::inTaskWithPhysicalMask(kernel_task, kIOMemoryUnshared | kIODirectionInOut, PAGE_SIZE, kUHCIStructureAllocationPhysicalMask);
+		if (!_cbiAlignBuffer)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not get alignment buffer", this);
+			status = kIOReturnNoMemory;
+			break;
+		}
+		status = _cbiAlignBuffer->prepare();
+		if (status)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - _alignBuffer->prepare failed with status(%p)", this, (void*)status);
+			break;
+		}
+		alignBufferPrepared = true;
+		status = dmaCommand->setMemoryDescriptor(_cbiAlignBuffer);
+		if (status)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - setMemoryDescriptor (_alignBuffer) returned err (%p)", this, (void*)status);
+			break;
+		}
+		
+		logicalBytes = (char*)_cbiAlignBuffer->getBytesNoCopy();
+		
+		offset = 0;
+		segments.fIOVMAddr = 0;
+		segments.fLength = 0;
+		numSegments = 1;
+		
+		status = dmaCommand->gen32IOVMSegments(&offset, &segments, &numSegments);
+		if (status || (numSegments != 1) || (segments.fLength != PAGE_SIZE))
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not generate segments err (%p) numSegments (%d) fLength (%d)", this, (void*)status, (int)numSegments, (int)segments.fLength);
+			dmaCommand->clearMemoryDescriptor();
+			status = status ? status : kIOReturnInternalError;
+			break;
+		}
+		pPhysical = segments.fIOVMAddr;
+		for (i=0; i < (PAGE_SIZE/kUHCI_BUFFER_CBI_ALIGN_SIZE); i++)
+		{
+			alignBuf = new UHCIAlignmentBuffer;
+			if (!alignBuf)
+			{
+				USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - unable to allocate expected UHCIAlignmentBuffer", this);
+				break;
+			}
+			alignBuf->paddr = pPhysical+(i*kUHCI_BUFFER_CBI_ALIGN_SIZE);
+			alignBuf->vaddr = (IOVirtualAddress)(logicalBytes+(i*kUHCI_BUFFER_CBI_ALIGN_SIZE));
+			alignBuf->userBuffer = NULL;
+			alignBuf->userOffset = 0;
+			alignBuf->type = UHCIAlignmentBuffer::kTypeCBI;
+			queue_enter(&_cbiAlignmentBuffers, alignBuf, UHCIAlignmentBuffer *, chain);
+		}
+		dmaCommand->clearMemoryDescriptor();
+		
+		// Set up some alignment buffers for isoch.  Note that each isoch transfer can be up to a max for 1023 bytes, so each alignment buffer needs to be
+		// at least that much -- we make them 1024 bytes.  We allocate kUHCI_BUFFER_ISOCH_ALIGN_QTY buffers to begin with.
+		
+		_isochAlignBuffer = IOBufferMemoryDescriptor::inTaskWithPhysicalMask(kernel_task, kIOMemoryUnshared | kIODirectionInOut, kUHCI_BUFFER_ISOCH_ALIGN_QTY * kUHCI_BUFFER_ISOCH_ALIGN_SIZE, kUHCIStructureAllocationPhysicalMask);
+		if (!_isochAlignBuffer)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not get isoch alignment buffer", this);
+			status = kIOReturnNoMemory;
+			break;
+		}
+		status = _isochAlignBuffer->prepare();
+		if (status)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - _alignBuffer->prepare failed with status(%p)", this, (void*)status);
+			break;
+		}
+		isochBufferPrepared = true;
+		status = dmaCommand->setMemoryDescriptor(_isochAlignBuffer);
+		if (status)
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - setMemoryDescriptor (_alignBuffer) returned err (%p)", this, (void*)status);
+			break;
+		}
+		
+		logicalBytes = (char*)_isochAlignBuffer->getBytesNoCopy();
+		
+		for (j=0; j < (kUHCI_BUFFER_ISOCH_ALIGN_QTY * kUHCI_BUFFER_ISOCH_ALIGN_SIZE / PAGE_SIZE) ; j++ )
+		{
+			offset = j * PAGE_SIZE;
+			segments.fIOVMAddr = 0;
+			segments.fLength = 0;
+			numSegments = 1;
+			
+			status = dmaCommand->gen32IOVMSegments(&offset, &segments, &numSegments);
+			if (status || (numSegments != 1) || (segments.fLength != PAGE_SIZE))
+			{
+				USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - could not generate segments err (%p) numSegments (%d) fLength (%d)", this, (void*)status, (int)numSegments, (int)segments.fLength);
+				dmaCommand->clearMemoryDescriptor();
+				status = status ? status : kIOReturnInternalError;
+				break;
+			}
+			pPhysical = segments.fIOVMAddr;
+			for (i=0; i < (PAGE_SIZE/kUHCI_BUFFER_ISOCH_ALIGN_SIZE); i++)
+			{
+				alignBuf = new UHCIAlignmentBuffer;
+				if (!alignBuf)
+				{
+					USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - unable to allocate expected UHCIAlignmentBuffer", this);
+					break;
+				}
+				alignBuf->paddr = pPhysical+(i*kUHCI_BUFFER_ISOCH_ALIGN_SIZE);
+				alignBuf->vaddr = (IOVirtualAddress)(logicalBytes + (j*PAGE_SIZE) + (i*kUHCI_BUFFER_ISOCH_ALIGN_SIZE));
+				alignBuf->userBuffer = NULL;
+				alignBuf->userOffset = 0;
+				alignBuf->type = UHCIAlignmentBuffer::kTypeIsoch;
+				queue_enter(&_isochAlignmentBuffers, alignBuf, UHCIAlignmentBuffer *, chain);
+			}
+		}
+		dmaCommand->clearMemoryDescriptor();
+		
+	} while (false);
+	
+	if (status)
+	{
+		if (_frameListBuffer)
+		{
+			if (frameBufferPrepared)
+				_frameListBuffer->complete();
+			_frameListBuffer->release();
+			_frameListBuffer = NULL;
+		}
+		if (_cbiAlignBuffer)
+		{
+			if (alignBufferPrepared)
+				_cbiAlignBuffer->complete();
+			_cbiAlignBuffer->release();
+			_cbiAlignBuffer = NULL;
+		}
+		if (_isochAlignBuffer)
+		{
+			if (alignBufferPrepared)
+				_isochAlignBuffer->complete();
+			_isochAlignBuffer->release();
+			_isochAlignBuffer = NULL;
+		}
+	}
+	
+	if (dmaCommand)
+	{
+		if (dmaCommand->getMemoryDescriptor())
+		{
+			USBError(1, "AppleUSBUHCI[%p]::InitializeBufferMemory - dmaCommand still has memory descriptor (%p)", this, dmaCommand->getMemoryDescriptor());
+			dmaCommand->clearMemoryDescriptor();
+		}
+		dmaCommand->release();
+	}
+	return status;
+}
+
+
+void
+AppleUSBUHCI::FreeBufferMemory()
+{
+	UHCIAlignmentBuffer			*ap;
+	
+	while (!queue_empty(&_cbiAlignmentBuffers)) 
+	{
+		queue_remove_first(&_cbiAlignmentBuffers, ap, UHCIAlignmentBuffer *, chain);
+		ap->release();
+	}
+	
+	while (!queue_empty(&_isochAlignmentBuffers)) 
+	{
+		queue_remove_first(&_isochAlignmentBuffers, ap, UHCIAlignmentBuffer *, chain);
+		ap->release();
+	}
+	
+	if (_frameListBuffer)
+	{
+		_frameListBuffer->complete();
+		_frameListBuffer->release();
+		_frameListBuffer = NULL;
+	}
+	if (_cbiAlignBuffer)
+	{
+		_cbiAlignBuffer->complete();
+		_cbiAlignBuffer->release();
+		_cbiAlignBuffer = NULL;
+	}
+	if (_isochAlignBuffer)
+	{
+		_isochAlignBuffer->complete();
+		_isochAlignBuffer->release();
+		_isochAlignBuffer = NULL;
+	}
+}
+
+UHCIAlignmentBuffer *
+AppleUSBUHCI::GetCBIAlignmentBuffer()
+{
+	UHCIAlignmentBuffer			*ap;
+	UInt32						align;
+	
+	if (queue_empty(&_cbiAlignmentBuffers)) 
+	{
+		USBError(1, "AppleUSBUHCI[%p]::GetCBIAlignmentBuffer - ran out of alignment buffers", this);
+		return NULL;
+	}
+	queue_remove_first(&_cbiAlignmentBuffers, ap, UHCIAlignmentBuffer *, chain);
+	ap->userBuffer = NULL;
+	ap->userOffset = 0;
+	ap->controller = this;
+	return ap;
+}
+
+
+void
+AppleUSBUHCI::ReleaseCBIAlignmentBuffer(UHCIAlignmentBuffer *ap)
+{
+	// USBLog(7, "AppleUSBUHCI[%p]::ReleaseAlignmentBuffer - putting alignment buffer %p into freeBuffers", this, ap);
+	queue_enter(&_cbiAlignmentBuffers, ap, UHCIAlignmentBuffer *, chain);
+}
+
+
+UHCIAlignmentBuffer *
+AppleUSBUHCI::GetIsochAlignmentBuffer()
+{
+	UHCIAlignmentBuffer			*ap;
+	UInt32						align;
+	
+	if (queue_empty(&_isochAlignmentBuffers)) 
+	{
+		USBError(1, "AppleUSBUHCI[%p]::GetIsochAlignmentBuffer - ran out of alignment buffers", this);
+		return NULL;
+	}
+	queue_remove_first(&_isochAlignmentBuffers, ap, UHCIAlignmentBuffer *, chain);
+	ap->userBuffer = NULL;
+	ap->userOffset = 0;
+	ap->controller = this;
+	
+	_uhciAlignmentBuffersInUse++;
+	if ( _uhciAlignmentBuffersInUse > _uhciAlignmentHighWaterMark )
+	{
+		_uhciAlignmentHighWaterMark++;
+		setProperty("AlignmentBuffersHighWaterMark", _uhciAlignmentHighWaterMark, 32);
+		USBLog(5, "AppleUSBUHCI[%p]::GetIsochAlignmentBuffer - New isoch alignment high water mark: %ld", this, _uhciAlignmentHighWaterMark);
+	}
+	
+	return ap;
+}
+
+
+void
+AppleUSBUHCI::ReleaseIsochAlignmentBuffer(UHCIAlignmentBuffer *ap)
+{
+	//USBLog(6, "AppleUSBUHCI[%p]::ReleaseIsochAlignmentBuffer - putting alignment buffer %p into freeBuffers", this, ap);
+	queue_enter(&_isochAlignmentBuffers, ap, UHCIAlignmentBuffer *, chain);
+	_uhciAlignmentBuffersInUse--;
+}
+
+
+OSDefineMetaClassAndStructors(UHCIAlignmentBuffer, OSObject);
+
+// ========================================================================
+#pragma mark AppleUSBUHCIDMACommand
+// ========================================================================
+OSDefineMetaClassAndStructors(AppleUSBUHCIDMACommand, IODMACommand)
+
+AppleUSBUHCIDMACommand *
+AppleUSBUHCIDMACommand::withSpecification(SegmentFunction outSegFunc,
+											UInt8           numAddressBits,
+											UInt64          maxSegmentSize,
+											MappingOptions  mappingOptions,
+											UInt64          maxTransferSize,
+											UInt32          alignment,
+											IOMapper       *mapper,
+											void           *refCon)
+{
+    AppleUSBUHCIDMACommand * me = new AppleUSBUHCIDMACommand;
+	
+    if (me && !me->initWithSpecification(outSegFunc,
+										 numAddressBits, maxSegmentSize,
+										 mappingOptions, maxTransferSize,
+										 alignment,      mapper, refCon))
+    {
+        me->release();
+        return NULL;
+    };
+	
+	queue_init(&me->_alignment_buffers);
+
+    return me;
+}
+
+
+
+IOReturn
+AppleUSBUHCIDMACommand::clearMemoryDescriptor(bool autoComplete)
+{
+	UHCIAlignmentBuffer			*ap;
+	IOReturn					ret;
+	
+	ret = IODMACommand::clearMemoryDescriptor(autoComplete);
+	while (!queue_empty(&_alignment_buffers)) 
+	{
+		queue_remove_first(&_alignment_buffers, ap, UHCIAlignmentBuffer *, chain);
+		USBLog(6, "AppleUSBUHCIDMACommand[%p]::clearMemoryDescriptor - got UHCIAlignmentBuffer (%p) paddr (%p) CBP (%p)", this, ap, (void*)ap->paddr, ap->userBuffer);
+		if (ap->actCount)
+		{
+			USBLog(6, "AppleUSBUHCIDMACommand[%p]::clearMemoryDescriptor - copying (%d) bytes from virtual address (%p)", this, (int)ap->actCount, (void*)ap->vaddr);
+			ap->userBuffer->writeBytes(ap->userOffset, (void*)ap->vaddr, ap->actCount);
+		}
+		if (ap->type == UHCIAlignmentBuffer::kTypeCBI)
+			ap->controller->ReleaseCBIAlignmentBuffer(ap);
+		else
+			ap->controller->ReleaseIsochAlignmentBuffer(ap);
+	}
+	return ret;
+}
+
+
+
+IODMACommand*
+AppleUSBUHCI::GetNewDMACommand()
+{
+	// our output function uses 64 bits, even though the controller can only handle 32 bits
+	return AppleUSBUHCIDMACommand::withSpecification(kIODMACommandOutputHost64, 32, 0);
+}
+
 // ========================================================================
 #pragma mark Debugging
 // ========================================================================
