@@ -57,6 +57,8 @@ class AppleUHCIQueueHead;
 class AppleUHCITransferDescriptor;
 class AppleUHCIIsochTransferDescriptor;
 class AppleUHCIIsochEndpoint;
+class AppleUSBUHCI;
+class AppleUSBUHCIDMACommand;
 
 // Convert USBLog to use kprintf debugging
 #define UHCI_USE_KPRINTF 0
@@ -146,29 +148,61 @@ enum {
  * The size of the buffer is the maxPacketSize
  * of the associated endpoint.
  */
-typedef struct UHCIAlignmentBuffer {
+class UHCIAlignmentBuffer : public OSObject
+{
+	OSDeclareDefaultStructors(UHCIAlignmentBuffer)
+
+public:
+	
+	enum bufferType
+	{
+		kTypeCBI,
+		kTypeIsoch
+	};
+	
     IOPhysicalAddress					paddr;
     IOVirtualAddress					vaddr;
 	
-    // For standard transfers
     IOMemoryDescriptor					*userBuffer;
     IOByteCount							userOffset;
-	
-    // For isochronous transfers
-    IOVirtualAddress					userAddr;
-    IOByteCount							userLength;
-
-    // Fields used by filter interrupt routine
-    UHCIAlignmentBuffer					*next;
-    UInt32								frameNumber;
-    UInt32								copyAtInterruptTime;
+	IOByteCount							actCount;
+	AppleUSBUHCI						*controller;
+	AppleUSBUHCIDMACommand				*dmaCommand;
+	bufferType							type;
 	
     // Queue fields
     queue_chain_t						chain;
-} UHCIAlignmentBuffer;
+};
+
+
+class AppleUSBUHCIDMACommand : public IODMACommand
+{
+    OSDeclareDefaultStructors(AppleUSBUHCIDMACommand)
+	
+public:
+	
+	queue_head_t			_alignment_buffers;
+
+	// constructor
+    static AppleUSBUHCIDMACommand *		withSpecification(SegmentFunction  outSegFunc,
+														  UInt8            numAddressBits,
+														  UInt64           maxSegmentSize,
+														  MappingOptions   mappingOptions = kMapped,
+														  UInt64           maxTransferSize = 0,
+														  UInt32           alignment = 1,
+														  IOMapper        *mapper = 0,
+														  void            *refCon = 0);
+	
+	// overriden methods
+    virtual IOReturn		clearMemoryDescriptor(bool autoComplete = true);
+
+};
+
 
 enum {
-    kUHCI_BUFFER_ALIGN = 4
+    kUHCI_BUFFER_CBI_ALIGN_SIZE		= 64,
+	kUHCI_BUFFER_ISOCH_ALIGN_SIZE	= 1024,
+	kUHCI_BUFFER_ISOCH_ALIGN_QTY	= 24
 };
 
 /* Checking for idleness.
@@ -256,6 +290,9 @@ protected:
     IOMemoryMap						*_ioMap;
     IOPhysicalAddress				_ioPhysAddress;
     IOVirtualAddress				_ioVirtAddress;
+	IOBufferMemoryDescriptor		*_frameListBuffer;
+	IOBufferMemoryDescriptor		*_cbiAlignBuffer;
+	IOBufferMemoryDescriptor		*_isochAlignBuffer;
     UInt16							_ioBase;
     UInt16							_vendorID;
     UInt16							_deviceID;
@@ -270,9 +307,11 @@ protected:
 	bool							_needToCreateRootHub;
 	bool							_wakingFromHibernation;
     
-    // Allocation of QHs and TDs 
-    queue_head_t					_allocatedBuffers;
-    
+	queue_head_t					_cbiAlignmentBuffers;			// alignment buffers for control/bulk/interrupt (64 byte buffers)
+	queue_head_t					_isochAlignmentBuffers;			// alignment buffers for isoch (1024 byte buffers)
+	SInt32							_uhciAlignmentHighWaterMark;
+	SInt32							_uhciAlignmentBuffersInUse;
+	
     // Timeouts
     AbsoluteTime					_lastTime;
     
@@ -284,9 +323,6 @@ protected:
     AbsoluteTime					_lastFrameNumberTime;
     UInt64							_lastTimeoutFrameNumber;
     
-    // Copying buffers at interrupt time
-    UHCIAlignmentBuffer				*_interruptAlignmentBuffers;
-
     // Interrupt status bits
     UInt32							_intrStatus;
 	UInt16							_hostControllerProcessInterrupt;
@@ -370,6 +406,7 @@ protected:
     void							RHEnablePort(int port, bool enable);
     IOReturn						RHSuspendPort(int port, bool suspend);
     IOReturn						RHResetPort(int port);
+	IOReturn						RHHoldPortReset(int port);
     
     void							RHDumpPortStatus(int port); // Debugging.
     void							RHDumpHubPortStatus(IOUSBHubPortStatus *status);
@@ -409,6 +446,7 @@ protected:
     // Initializing hardware and setup
     IOReturn						HardwareInit(void);
     IOReturn						Run(bool);
+	IOReturn						InitializeAlignmentBuffers(void);
     
     // Memory management
 	AppleUHCITransferDescriptor			*AllocateTD(AppleUHCIQueueHead *);
@@ -443,30 +481,23 @@ protected:
     /*
      * Isochronous support.
      */
-    IOReturn								CreateIsochTransfer(	IOUSBControllerIsochEndpoint*		pEP,
-																	IOUSBIsocCompletion					completion,
-																	UInt64								frameNumberStart,
-																	IOMemoryDescriptor					*pBuffer,
-																	UInt32								frameCount,
-																	IOUSBIsocFrame						*pFrames,				// could be IOUSBLowLatencyIsocFrame*
-																	UInt32								updateFrequency = 0,
-																	bool								lowLatency = false,
-																	bool								requestFromRosettaClient = false);
-    
+    IOReturn								CreateIsochTransfer(IOUSBControllerIsochEndpoint* pEP, IOUSBIsocCommand *command);    
         
-	void							AddIsochFramesToSchedule(IOUSBControllerIsochEndpoint*);
-    IOReturn						AbortIsochEP(IOUSBControllerIsochEndpoint*);
-    IOReturn						DeleteIsochEP(IOUSBControllerIsochEndpoint*);
+	void									AddIsochFramesToSchedule(IOUSBControllerIsochEndpoint*);
+    IOReturn								AbortIsochEP(IOUSBControllerIsochEndpoint*);
+    IOReturn								DeleteIsochEP(IOUSBControllerIsochEndpoint*);
 
 	/*
      * Power management.
      */
     bool									_unloadUIMAcrossSleep;
+    bool									_remoteWakeupOccurred;
     UInt32									_saveFrameAddress;
     UInt16									_saveFrameNumber;
 	UInt16									_saveInterrupts;
-    bool									_remoteWakeupOccurred;
     IONotifier								*_powerDownNotifier;
+	UInt32									_ExpressCardPort;					// Port number of ExpressCard (0 if no ExpressCard on this controller)
+	bool									_badExpressCardAttached;			// True if a driver has identified a bad ExpressCard
     
     virtual void							initForPM (IOPCIDevice *provider);
     unsigned long							maxCapabilityForDomainState ( IOPMPowerFlags domainState );
@@ -475,6 +506,8 @@ protected:
 																	void *messageArgument, vm_size_t argSize);
     
     virtual IOReturn						setPowerState( unsigned long powerStateOrdinal, IOService* whatDevice );
+	
+	UInt32									ExpressCardPort( IOService * provider );
     
     void									ResumeController(void);
     void									SuspendController(void);
@@ -621,6 +654,7 @@ public:
 																USBDeviceAddress highSpeedHub,
 																int      highSpeedPort);
 
+	// obsolete method
     virtual IOReturn					UIMCreateIsochTransfer( short						functionAddress,
 																short						endpointNumber,
 																IOUSBIsocCompletion			completion,
@@ -630,6 +664,7 @@ public:
 																UInt32						frameCount,
 																IOUSBIsocFrame				*pFrames);
 
+	// obsolete method
     virtual IOReturn					UIMCreateIsochTransfer(short						functionAddress,
 															   short						endpointNumber,
 															   IOUSBIsocCompletion			completion,
@@ -640,6 +675,9 @@ public:
 															   IOUSBLowLatencyIsocFrame		*pFrames,
 															   UInt32						updateFrequency);
         
+	// new method
+	virtual IOReturn					UIMCreateIsochTransfer(IOUSBIsocCommand *command);
+
     virtual IOReturn UIMAbortEndpoint(short functionNumber, short endpointNumber, short direction);
     virtual IOReturn UIMDeleteEndpoint(short functionNumber, short endpointNumber, short direction);
     virtual IOReturn UIMClearEndpointStall(short functionNumber, short endpointNumber, short direction);
@@ -679,21 +717,22 @@ public:
 							   IOReturn							err);
     
     UInt32 findBufferRemaining(AppleUHCIQueueHead *pQH);
+	
+	// alignment buffers
+	UHCIAlignmentBuffer *						GetCBIAlignmentBuffer();
+	void										ReleaseCBIAlignmentBuffer(UHCIAlignmentBuffer*);
+	UHCIAlignmentBuffer *						GetIsochAlignmentBuffer();
+	void										ReleaseIsochAlignmentBuffer(UHCIAlignmentBuffer*);
+
+	IOReturn									InitializeBufferMemory();
+	void										FreeBufferMemory();
+
     
 	virtual IOUSBControllerIsochEndpoint*			AllocateIsochEP();
 
+    virtual IOReturn								GetLowLatencyOptionsAndPhysicalMask(IOOptionBits *optionBits, mach_vm_address_t *physicalMask);
+	virtual IODMACommand							*GetNewDMACommand();
+    virtual void									PutTDonDoneQueue(IOUSBControllerIsochEndpoint* pED, IOUSBControllerIsochListElement *pTD, bool checkDeferred);
 };
 
-class UHCIMemoryBuffer : public IOBufferMemoryDescriptor
-{
-    OSDeclareDefaultStructors(UHCIMemoryBuffer)
-
-protected:
-    virtual void free();
-public:
-    queue_chain_t _chain;
-    static UHCIMemoryBuffer *newBuffer(bool dmaable = true);
-};
-
-
 #endif /* ! _IOKIT_AppleUSBUHCI_H */
