@@ -40,8 +40,8 @@ IOUSBCommand::NewCommand()
     
     if (me && !me->init())
     {
-	me->release();
-	me = NULL;
+		me->release();
+		me = NULL;
     }
     return me;
 }
@@ -56,13 +56,13 @@ IOUSBCommand::init()
     // allocate our expansion data
     if (!_expansionData)
     {
-	_expansionData = (ExpansionData *)IOMalloc(sizeof(ExpansionData));
-	if (!_expansionData)
-	    return false;
-	bzero(_expansionData, sizeof(ExpansionData));
+		_expansionData = (ExpansionData *)IOMalloc(sizeof(ExpansionData));
+		if (!_expansionData)
+			return false;
+		bzero(_expansionData, sizeof(ExpansionData));
     }
     return true;
- }
+}
 
 
 void 
@@ -73,10 +73,10 @@ IOUSBCommand::free()
     //
     if (_expansionData)
     {
-	IOFree(_expansionData, sizeof(ExpansionData));
+		IOFree(_expansionData, sizeof(ExpansionData));
         _expansionData = NULL;
     }
-
+	
     super::free();
 }
 
@@ -144,19 +144,28 @@ IOUSBCommand::SetClientCompletion(IOUSBCompletion completion)
 void 
 IOUSBCommand::SetDataRemaining(UInt32 dr) 
 {
-    _dataRemaining = dr;
+	if (_expansionData->_masterUSBCommand)
+		_expansionData->_masterUSBCommand->_dataRemaining = dr;
+	else
+		_dataRemaining = dr;
 }
 
 void 
 IOUSBCommand::SetStage(UInt8 stage) 
 {
-    _stage = stage;
+	if (_expansionData->_masterUSBCommand)
+		_expansionData->_masterUSBCommand->_stage = stage;
+	else
+		_stage = stage;
 }
 
 void 
 IOUSBCommand::SetStatus(IOReturn stat) 
 {
-    _status = stat;
+	if (_expansionData->_masterUSBCommand)
+		_expansionData->_masterUSBCommand->_status = stat;
+	else
+		_status = stat;
 }
 
 void 
@@ -192,8 +201,11 @@ IOUSBCommand::SetCompletionTimeout(UInt32 to)
 void 
 IOUSBCommand::SetUIMScratch(UInt32 index, UInt32 value) 
 { 
-    if (index < 10) 
-        _UIMScratch[index] = value;
+    if (index < 10)
+		if (_expansionData->_masterUSBCommand)
+			_expansionData->_masterUSBCommand->_UIMScratch[index] = value;
+		else
+			_UIMScratch[index] = value;
 }
 
 void 
@@ -248,143 +260,185 @@ IOUSBCommand::SetIsSyncTransfer(bool isSync)
 }
 
 
+void					
+IOUSBCommand::SetBufferUSBCommand(IOUSBCommand *bufferUSBCommand)
+{
+	if (!bufferUSBCommand && (_expansionData->_bufferUSBCommand))
+		_expansionData->_bufferUSBCommand->_expansionData->_masterUSBCommand = NULL;
+	
+	_expansionData->_bufferUSBCommand = bufferUSBCommand;
+	
+	if (bufferUSBCommand)
+		bufferUSBCommand->_expansionData->_masterUSBCommand = this; 
+}
+
+
 usbCommand 
 IOUSBCommand::GetSelector(void) 
 {
-    return _selector;
+	// This one can be different for the two command (master and buffer)
+	return _selector;
 }
 
 IOUSBDeviceRequestPtr 
 IOUSBCommand::GetRequest(void) 
 {
-    return _request;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_request : _request;
 }
 
-USBDeviceAddress IOUSBCommand::GetAddress(void) 
+USBDeviceAddress 
+IOUSBCommand::GetAddress(void) 
 {
-    return _address;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_address : _address;
 }
 
-UInt8 IOUSBCommand::GetEndpoint(void) 
+UInt8 
+IOUSBCommand::GetEndpoint(void) 
 {
-    return _endpoint;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_endpoint : _endpoint;
 }
 
-UInt8 IOUSBCommand::GetDirection(void) 
+UInt8 
+IOUSBCommand::GetDirection(void) 
 {
-    return _direction;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_direction : _direction;
 }
 
-UInt8 IOUSBCommand::GetType(void) 
+UInt8 
+IOUSBCommand::GetType(void) 
 {
-    return _type;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_type : _type;
 }
 
-bool IOUSBCommand::GetBufferRounding(void) 
+bool 
+IOUSBCommand::GetBufferRounding(void) 
 {
     return _bufferRounding;
 }
 
-IOMemoryDescriptor* IOUSBCommand::GetBuffer(void) 
+IOMemoryDescriptor* 
+IOUSBCommand::GetBuffer(void) 
 { 
-    return _buffer;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_buffer : _buffer;
 }
 
-IOUSBCompletion IOUSBCommand::GetUSLCompletion(void) 
+IOUSBCompletion 
+IOUSBCommand::GetUSLCompletion(void) 
 { 
-    return _uslCompletion;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_uslCompletion : _uslCompletion;
 }
 
-IOUSBCompletion IOUSBCommand::GetClientCompletion(void) 
+IOUSBCompletion 
+IOUSBCommand::GetClientCompletion(void) 
 { 
-    return _clientCompletion;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_clientCompletion : _clientCompletion;
 }
 
-UInt32 IOUSBCommand::GetDataRemaining(void) 
+UInt32 
+IOUSBCommand::GetDataRemaining(void) 
 { 
-    return _dataRemaining;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_dataRemaining : _dataRemaining;
 }
 
-UInt8 IOUSBCommand::GetStage(void) 
+UInt8 
+IOUSBCommand::GetStage(void) 
 { 
-    return _stage;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_stage : _stage;
 }
 
-IOReturn IOUSBCommand::GetStatus(void) 
+IOReturn 
+IOUSBCommand::GetStatus(void) 
 { 
-    return _status;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_status : _status;
 }
 
-IOMemoryDescriptor * IOUSBCommand::GetOrigBuffer(void) 
+IOMemoryDescriptor * 
+IOUSBCommand::GetOrigBuffer(void) 
 { 
-    return _origBuffer;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_origBuffer : _origBuffer;
 }
 
-IOUSBCompletion IOUSBCommand::GetDisjointCompletion(void) 
+IOUSBCompletion 
+IOUSBCommand::GetDisjointCompletion(void) 
 { 
-    return _disjointCompletion;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_disjointCompletion : _disjointCompletion;
 }
 
-IOByteCount IOUSBCommand::GetDblBufLength(void) 
+IOByteCount 
+IOUSBCommand::GetDblBufLength(void) 
 { 
-    return _dblBufLength;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_dblBufLength : _dblBufLength;
 }
 
-UInt32 IOUSBCommand::GetNoDataTimeout(void) 
+UInt32 
+IOUSBCommand::GetNoDataTimeout(void) 
 { 
-    return _noDataTimeout;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_noDataTimeout : _noDataTimeout;
 }
 
-UInt32 IOUSBCommand::GetCompletionTimeout(void) 
+UInt32 
+IOUSBCommand::GetCompletionTimeout(void) 
 {
-    return _completionTimeout;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_completionTimeout : _completionTimeout;
 }
 
 UInt32 IOUSBCommand::GetUIMScratch(UInt32 index) 
 { 
-    return (index < 10) ? _UIMScratch[index] : 0;
+	if (index < 10)
+		return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_UIMScratch[index] : _UIMScratch[index];
+	else
+		return 0;
 }
- 
-IOByteCount IOUSBCommand::GetReqCount(void) 
+
+IOByteCount 
+IOUSBCommand::GetReqCount(void) 
 { 
     return _expansionData->_reqCount;
 }
 
-IOMemoryDescriptor *IOUSBCommand::GetRequestMemoryDescriptor(void)
+IOMemoryDescriptor*
+IOUSBCommand::GetRequestMemoryDescriptor(void)
 {
-    return _expansionData->_requestMemoryDescriptor;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_expansionData->_requestMemoryDescriptor : _expansionData->_requestMemoryDescriptor;
 }
 
-IOMemoryDescriptor *IOUSBCommand::GetBufferMemoryDescriptor(void)
+// this one is different in that the buffer command (the child) will use this for its own memory descriptor
+IOMemoryDescriptor*
+IOUSBCommand::GetBufferMemoryDescriptor(void)
 {
     return _expansionData->_bufferMemoryDescriptor;
 }
 
 
-bool IOUSBCommand::GetMultiTransferTransaction(void)
+bool 
+IOUSBCommand::GetMultiTransferTransaction(void)
 {
-    return _expansionData->_multiTransferTransaction;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_expansionData->_multiTransferTransaction : _expansionData->_multiTransferTransaction;
 }
 
 
-bool IOUSBCommand::GetFinalTransferInTransaction(void)
+bool 
+IOUSBCommand::GetFinalTransferInTransaction(void)
 {
-    return _expansionData->_finalTransferInTransaction;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_expansionData->_finalTransferInTransaction : _expansionData->_finalTransferInTransaction;
 }
 
-bool IOUSBCommand::GetUseTimeStamp(void)
+bool 
+IOUSBCommand::GetUseTimeStamp(void)
 {
-    return _expansionData->_useTimeStamp;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_expansionData->_useTimeStamp : _expansionData->_useTimeStamp;
 }
 
-AbsoluteTime IOUSBCommand::GetTimeStamp(void)
+AbsoluteTime 
+IOUSBCommand::GetTimeStamp(void)
 {
-    return _expansionData->_timeStamp;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_expansionData->_timeStamp : _expansionData->_timeStamp;
 }
 
-bool IOUSBCommand::GetIsSyncTransfer(void)
+bool 
+IOUSBCommand::GetIsSyncTransfer(void)
 {
-    return _expansionData->_isSyncTransfer;
+	return _expansionData->_masterUSBCommand ? _expansionData->_masterUSBCommand->_expansionData->_isSyncTransfer : _expansionData->_isSyncTransfer;
 }
 
 
@@ -395,8 +449,8 @@ IOUSBIsocCommand::NewCommand()
     
     if (me && !me->init())
     {
-	me->release();
-	me = NULL;
+		me->release();
+		me = NULL;
     }
     return me;
 }
@@ -409,13 +463,13 @@ IOUSBIsocCommand::init()
     // allocate our expansion data
     if (!_expansionData)
     {
-	_expansionData = (ExpansionData *)IOMalloc(sizeof(ExpansionData));
-	if (!_expansionData)
-	    return false;
-	bzero(_expansionData, sizeof(ExpansionData));
+		_expansionData = (ExpansionData *)IOMalloc(sizeof(ExpansionData));
+		if (!_expansionData)
+			return false;
+		bzero(_expansionData, sizeof(ExpansionData));
     }
     return true;
- }
+}
 
 
 void 
@@ -426,7 +480,7 @@ IOUSBIsocCommand::free()
     //
     if (_expansionData)
     {
-	IOFree(_expansionData, sizeof(ExpansionData));
+		IOFree(_expansionData, sizeof(ExpansionData));
         _expansionData = NULL;
     }
     super::free();
