@@ -29,11 +29,24 @@
 //
 //================================================================================================
 //
+#include <IOKit/IOBufferMemoryDescriptor.h>
+
 #include <IOKit/usb/IOUSBController.h>
 #include <IOKit/usb/IOUSBControllerV2.h>
 #include <IOKit/usb/IOUSBLog.h>
 
 
+#define CONTROLLERV2_USE_KPRINTF 0
+
+#if CONTROLLERV2_USE_KPRINTF
+#undef USBLog
+#undef USBError
+void kprintf(const char *format, ...)
+__attribute__((format(printf, 1, 2)));
+#define USBLog( LEVEL, FORMAT, ARGS... )  if ((LEVEL) <= 3) { kprintf( FORMAT "\n", ## ARGS ) ; }
+#define USBError( LEVEL, FORMAT, ARGS... )  { kprintf( FORMAT "\n", ## ARGS ) ; }
+#endif
+
 extern IOReturn CheckForDisjointDescriptor(IOUSBCommand *command, UInt16 maxPacketSize);
 
 //================================================================================================
@@ -54,7 +67,6 @@ enum {
     kStatusBack = 0x40
 };
 
-#define USEDYNAMICCOMMAND 1
 //================================================================================================
 //
 //   IOKit Constructors and Destructors
@@ -130,21 +142,24 @@ IOUSBControllerV2::free()
 
 
 void
-IOUSBControllerV2::clearTTHandler( OSObject *	target,
-                                   void *	parameter,
-                                   IOReturn	status,
-                                   UInt32	bufferSizeRemaining)
+IOUSBControllerV2::clearTTHandler(OSObject *target, void *parameter, IOReturn status, UInt32	bufferSizeRemaining)
 {
-    IOUSBController *	me = (IOUSBController *)target;
-    IOUSBCommand *	command = (IOUSBCommand *)parameter;
-    UInt8		sent, back, todo;
-    UInt8		hubAddr = command->GetAddress();
+    IOUSBController *		me = (IOUSBController *)target;
+    IOUSBCommand *			command = (IOUSBCommand *)parameter;
+	IOMemoryDescriptor *	memDesc = NULL;
+	IODMACommand *			dmaCommand = NULL;
+    UInt8					sent, back, todo;
+    UInt8					hubAddr = command->GetAddress();
     
     USBLog(5,"clearTTHandler: status (0x%x)", status);
+	if (!me || !command)
+	{
+		USBError(1,"clearTTHandler: missing controller or command");
+	}
 	
     sent = (command->GetStage() & 0x0f) << 4;
     back = command->GetStage() & 0xf0;
-    todo = sent ^ back; /* thats xor */
+    todo = sent ^ back;						// thats xor
 	
     if((todo & kSetupBack) != 0)
     {
@@ -153,17 +168,39 @@ IOUSBControllerV2::clearTTHandler( OSObject *	target,
     }
     else
     {
+		dmaCommand = command->GetDMACommand();
+		memDesc = command->GetRequestMemoryDescriptor();
         command->SetStage(0);
-		USBLog(5, "%s[%p]::clearTTHandler - about to free IOUSBDevRequest (%p)", me->getName(), me, command->GetRequest());
-		IOFree(command->GetRequest(), sizeof(IOUSBDevRequest));
-#if USEDYNAMICCOMMAND
+		if (dmaCommand)
+		{
+			if (dmaCommand->getMemoryDescriptor())
+			{
+				if (dmaCommand->getMemoryDescriptor() != memDesc)
+				{
+					USBLog(1, "clearTTHandler: DMA Command Memory Descriptor (%p) does not match Request MemoryDescriptor (%p)", dmaCommand->getMemoryDescriptor(), memDesc);
+				}
+				USBLog(7, "clearTTHandler: clearing memory descriptor (%p) from dmaCommand (%p)", dmaCommand->getMemoryDescriptor(), dmaCommand);
+				dmaCommand->clearMemoryDescriptor();
+			}
+			else
+			{
+				USBLog(1, "clearTTHandler - dmaCommand (%p) already cleared", dmaCommand);
+			}
+		}
+		if (memDesc)
+		{
+			USBLog(1, "clearTTHandler - completing and freeing memory descriptor (%p)", memDesc);
+			memDesc->complete();
+			memDesc->release();
+		}
+		else
+		{
+			USBLog(1, "clearTTHandler - missing memory descriptor");
+		}
 		USBLog(5,"clearTTHandler: We've already seen the setup, deallocate command (%p)", command);
 		me->_freeUSBCommandPool->returnCommand(command);   
-#else
-		USBLog(5,"clearTTHandler: We've already seen the setup, using static command");
-#endif
     }
-    if (status != kIOReturnSuccess)
+    if ((status != kIOReturnSuccess) && (status != kIOUSBTransactionReturned))
     {
 		USBLog(1, "%s[%p]::clearTTHandler - error response from hub, clearing hub endpoint stall", me->getName(), me);
 		me->UIMClearEndpointStall(hubAddr, 0, kUSBAnyDirn);
@@ -175,13 +212,15 @@ OSMetaClassDefineReservedUsed(IOUSBControllerV2,  6);
 void 
 IOUSBControllerV2::ClearTT(USBDeviceAddress fnAddress, UInt8 endpt, Boolean IN)
 {
-    UInt16 		wValue;
-    IOUSBDevRequest 	*clearRequest;
-    short 		hubAddress;
-    IOUSBCommand 	*clearCommand;
-    IOUSBCompletion	completion;
-    int 		i;
-    IOReturn 		err;
+    UInt16						wValue;
+	IOBufferMemoryDescriptor	*memDesc;
+    IOUSBDevRequest				*clearRequest;
+    short						hubAddress;
+    IOUSBCommand				*clearCommand;
+    IOUSBCompletion				completion;
+    int							i;
+    IOReturn					err;
+	IODMACommand				*dmaCommand = NULL;
 	
     USBLog(5,"+%s[%p]::ClearTT", getName(), this);
     hubAddress = _highSpeedHub[fnAddress];	// Address of its controlling hub.
@@ -191,7 +230,13 @@ IOUSBControllerV2::ClearTT(USBDeviceAddress fnAddress, UInt8 endpt, Boolean IN)
 		return;
     }
 	
-#if USEDYNAMICCOMMAND
+	memDesc = IOBufferMemoryDescriptor::withOptions(kIOMemoryUnshared | kIODirectionInOut, sizeof(IOUSBDevRequest));
+	if (!memDesc)
+	{
+		USBLog(1,"%s[%p]::ClearTT Could not get a memory descriptor",getName(),this);
+		return;
+	}
+
     clearCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
     if ( clearCommand == NULL )
     {
@@ -201,40 +246,33 @@ IOUSBControllerV2::ClearTT(USBDeviceAddress fnAddress, UInt8 endpt, Boolean IN)
 		if ( clearCommand == NULL )
 		{
 			USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBCommand",getName(),this);
+			memDesc->release();
 			return;
 		}
     }
-    USBLog(6, "%s[%p]::ClearTT V2 got command (%p)", getName(), this, clearCommand);
-#else
-    clearCommand = _v2ExpansionData->ClearTTCommand;
-    if ( clearCommand == NULL )
-    {
-		_v2ExpansionData->ClearTTCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
-		clearCommand = _v2ExpansionData->ClearTTCommand;
-		if ( clearCommand == NULL )
-		{
-			USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBCommand",getName(),this);
-			return;
-		}
-		USBLog(6,"%s[%p]::ClearTT gto static IOUSBCommand",getName(),this);
-    }
-	
-    if(clearCommand->GetStage() != 0)
-    {
-		USBLog(1,"%s[%p]::ClearTT our command is in use, giving up",getName(),this);
+    USBLog(7, "%s[%p]::ClearTT V2 got command (%p)", getName(), this, clearCommand);
+
+	dmaCommand = clearCommand->GetDMACommand();
+	if (!dmaCommand)
+	{
+		USBError(1,"%s[%p]::ClearTT - No dmaCommand in the usb command", getName(), this);
 		return;
-    }
+	}
 	
-#endif
+	if (dmaCommand->getMemoryDescriptor())
+	{
+		IOMemoryDescriptor		*memDesc = (IOMemoryDescriptor *)dmaCommand->getMemoryDescriptor();
+		USBError(1,"%s[%p]::ClearTT - dmaCommand (%p) already had memory descriptor (%p) - clearing", getName(), this, dmaCommand, memDesc);
+		dmaCommand->clearMemoryDescriptor();
+		// memDesc->complete();					should i do this?
+		// memDesc->release();					should i do this?
+	}
 	
-    
-    clearRequest = (IOUSBDevRequest*)IOMalloc(sizeof(IOUSBDevRequest));
+    clearRequest = (IOUSBDevRequest*)memDesc->getBytesNoCopy();
     if (!clearRequest)
     {
 		USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBDevRequest", getName(), this);
-#if USEDYNAMICCOMMAND
 		_freeUSBCommandPool->returnCommand(clearCommand);
-#endif
 		return;
     }
     USBLog(5, "%s[%p]::ClearTT - got IOUSBDevRequest (%p)", getName(), this, clearRequest);
@@ -264,16 +302,16 @@ IOUSBControllerV2::ClearTT(USBDeviceAddress fnAddress, UInt8 endpt, Boolean IN)
 	
     clearRequest->bmRequestType = 0x23;
     clearRequest->bRequest = 8;
-    clearRequest->wValue = wValue;
+    clearRequest->wValue = HostToUSBWord(wValue);
     if(_v2ExpansionData->_multiTT[hubAddress])
     {  // MultiTT hub needs port address here
-		clearRequest->wIndex = _highSpeedPort[fnAddress];
+		clearRequest->wIndex = HostToUSBWord(_highSpeedPort[fnAddress]);
     }
     else
     {  // Single TT hubs need 1 here
-		clearRequest->wIndex = 1;
+		clearRequest->wIndex = HostToUSBWord(1);
     }
-    clearRequest->wLength = 0;
+    clearRequest->wLength = HostToUSBWord(0);
     clearRequest->pData = NULL;
     clearRequest->wLenDone = 0;
 	
@@ -296,7 +334,22 @@ IOUSBControllerV2::ClearTT(USBDeviceAddress fnAddress, UInt8 endpt, Boolean IN)
     clearCommand->SetNoDataTimeout(5000);
     clearCommand->SetCompletionTimeout(0);
     clearCommand->SetStage(0);
+	clearCommand->SetBufferUSBCommand(NULL);
+
+	memDesc->prepare();
+	clearCommand->SetRequestMemoryDescriptor(memDesc);
+	clearCommand->SetReqCount(8);
     
+	USBLog(7,"%s[%p]::ClearTT - setting memory descriptor (%p) into dmaCommand (%p)", getName(), this, memDesc, dmaCommand);
+	err = dmaCommand->setMemoryDescriptor(memDesc);
+	if (err)
+	{
+		USBError(1,"%s[%p]::ClearTT - err (%p) setting memory descriptor (%p) into dmaCommand (%p)", getName(), this, (void*)err, memDesc, dmaCommand);
+		memDesc->complete();
+		memDesc->release();
+		return;
+	}
+	
     for (i=0; i < 10; i++)
 		clearCommand->SetUIMScratch(i, 0);
 	
@@ -363,16 +416,50 @@ IOUSBControllerV2::DoCreateEP(OSObject *owner,
             break;
 			
         case kUSBIsoc:
-			USBLog(4, "%s[%p]::DoCreateEP - Creating Isoch EP with interval %d [raw %d]", me->getName(), me, (endpoint->interval == 4) ? 8 : 1, endpoint->interval);
-			// only support intervals of 1 and 8
-            err = me->UIMCreateIsochEndpoint(address,
+		  {
+			UInt32 interval;
+
+			if (speed == kUSBDeviceSpeedHigh)
+			  {
+
+				// Filter out cases that violate the USB spec:
+				if (endpoint->interval < 1 || endpoint->interval > 16)
+				  {
+					USBLog(1, "%s[%p]::DoCreateEP - The USB 2.0 spec only allows Isoch EP with bInterval values of 1 through 16 "
+						   "(see Table 9-13), but the illegal interval %d [0x%x] was requested, returning kIOReturnBadArgument", 
+						   me->getName(), me, endpoint->interval, endpoint->interval);
+					err = kIOReturnBadArgument;
+					break;
+				  }
+
+				interval = (1 << (endpoint->interval - 1));
+
+				USBLog(1, "%s[%p]::DoCreateEP - Creating a High-Speed Isoch EP with interval %u [raw %u]", me->getName(), me, 
+					   (unsigned int )interval, (unsigned int )endpoint->interval);
+
+			  }
+			else
+			  {	// Full speed devices may have an invalid bInterval, thinking it doesn't
+				// matter. To protect ourselves, assign an interval of 1 - our code will
+				// do the right thing for full-speed devices then.
+				interval = 1;
+
+				USBLog(4, "%s[%p]::DoCreateEP - Creating a Full-Speed Isoch EP with interval %u [raw %u]", me->getName(), me, 
+					   (unsigned int )interval, (unsigned int )endpoint->interval);
+
+			  }
+
+
+			err = me->UIMCreateIsochEndpoint(address,
 											 endpoint->number,
 											 endpoint->maxPacketSize,
 											 endpoint->direction,
 											 me->_highSpeedHub[address],
 											 me->_highSpeedPort[address],
-											 (endpoint->interval == 4) ? 8 : 1);
-            break;
+											 interval);
+			break;
+		  }
+
 			
         default:
             err = kIOReturnBadArgument;
@@ -537,12 +624,13 @@ OSMetaClassDefineReservedUsed(IOUSBControllerV2,  7);
 IOReturn
 IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address, Endpoint *endpoint, IOUSBCompletionWithTimeStamp *completion, UInt32 noDataTimeout, UInt32 completionTimeout, IOByteCount reqCount)
 {
-    IOReturn	 	err = kIOReturnSuccess;
-    IOUSBCommand 	*command;
-    IOUSBCompletion 	nullCompletion;
-    IOUSBCompletion 	theCompletion;
+    IOReturn				err = kIOReturnSuccess;
+    IOUSBCommand			*command;
+    IOUSBCompletion			nullCompletion;
+    IOUSBCompletion			theCompletion;
     IOUSBCompletionAction 	theAction;
-    int			i;
+	IODMACommand			*dmaCommand = NULL;
+    int						i;
 	
     USBLog(7, "%s[%p]::ReadV2 - reqCount = %ld", getName(), this, reqCount);
 	
@@ -590,6 +678,40 @@ IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address,
         }
     }
 	
+	if (reqCount)
+	{
+		dmaCommand = command->GetDMACommand();
+		
+		if (!dmaCommand)
+		{
+			USBError(1, "%s[%p]::ReadV2 - no DMA COMMAND", getName(), this);
+            return kIOReturnNoResources;
+		}
+		if (dmaCommand->getMemoryDescriptor())
+		{
+			IOMemoryDescriptor		*memDesc = (IOMemoryDescriptor *)dmaCommand->getMemoryDescriptor();
+			USBError(1, "%s[%p]::ReadV2 - dma command (%p) already contains memory descriptor (%p) - clearing", getName(), this, dmaCommand, memDesc);
+			dmaCommand->clearMemoryDescriptor();
+			// memDesc->complete();					should i do this?
+			// memDesc->release();					should i do this?
+		}
+		// err = buffer->prepare();			should i do this?
+		if (0) // if (err)
+		{
+			USBError(1, "%s[%p]::ReadV2 - err(%p) attempting to prepare buffer (%p)", getName(), this, (void*)err, buffer);
+			return err;
+		}
+		USBLog(7, "%s[%p]::ReadV2 - setting memory descriptor (%p) into dmaCommand (%p)", getName(), this, buffer, dmaCommand);
+		err = dmaCommand->setMemoryDescriptor(buffer);
+		if (err)
+		{
+			USBError(1, "%s[%p]::ReadV2 - err(%p) attempting to set the memory descriptor to the dmaCommand", getName(), this, (void*)err);
+			// buffer->complete();			should i do this?
+			_freeUSBCommandPool->returnCommand(command);
+			return err;
+		}
+	}
+	
     theCompletion.target = completion->target;
     theCompletion.action = (IOUSBCompletionAction)completion->action;
     theCompletion.parameter = completion->parameter;
@@ -629,15 +751,36 @@ IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address,
 		// If we have a sync request, then we always return the command after the DoIOTransfer.  If it's an async request, we only return it if 
 		// we get an immediate error
 		//
-		if ( command->GetIsSyncTransfer() ||  (!command->GetIsSyncTransfer() && (kIOReturnSuccess != err)) )
+		if ( command->GetIsSyncTransfer() || (kIOReturnSuccess != err) )
 		{
+			IODMACommand		*dmaCommand = command->GetDMACommand();
+			IOMemoryDescriptor	*memDesc = dmaCommand ? (IOMemoryDescriptor	*)dmaCommand->getMemoryDescriptor() : NULL;
+			
+			nullCompletion = command->GetDisjointCompletion();
+			if (nullCompletion.action)
+			{
+				USBLog(6, "%s[%p]::ReadV2 - SYNC xfer or immediate error with Disjoint Completion", getName(), this);
+			}
+			if (memDesc)
+			{
+				USBLog(7, "%s[%p]::ReadV2 - SYNC xfer or immediate error - clearing memDesc (%p) from dmaCommand (%p)", getName(), this, memDesc, dmaCommand);
+				dmaCommand->clearMemoryDescriptor();
+			}
 			_freeUSBCommandPool->returnCommand(command);
 		}
 	}
 	else
 	{
+		IODMACommand		*dmaCommand = command->GetDMACommand();
+		IOMemoryDescriptor	*memDesc = dmaCommand ? (IOMemoryDescriptor	*)dmaCommand->getMemoryDescriptor() : NULL;
+
 		// CheckFordDisjoint returned an error, so free up the comand
 		//
+		if (memDesc)
+		{
+			USBLog(7, "%s[%p]::ReadV2 - CheckForDisjointDescriptor error (%p) - clearing memDesc (%p) from dmaCommand (%p)", getName(), this, (void*)err, memDesc, dmaCommand);
+			dmaCommand->clearMemoryDescriptor();
+		}
 		_freeUSBCommandPool->returnCommand(command);
 	}
 	
@@ -917,7 +1060,8 @@ IOUSBControllerV2::ReturnIsochDoneQueue(IOUSBControllerIsochEndpoint* pEP)
 {
     IOUSBControllerIsochListElement		*pTD = GetTDfromDoneQueue(pEP);
     IOUSBIsocFrame						*pFrames = NULL;
-		
+	IOUSBIsocCompletionAction			pHandler;
+	
     USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue (%p)", this, pEP);
     if (pTD)
     {
@@ -932,7 +1076,6 @@ IOUSBControllerV2::ReturnIsochDoneQueue(IOUSBControllerIsochEndpoint* pEP)
 		USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue: TD %p", this, pTD);
 		if( pTD->_completion.action != NULL)
 		{
-			IOUSBIsocCompletionAction 	pHandler;
 			
 			pHandler = pTD->_completion.action;
 			pTD->_completion.action = NULL;
@@ -948,8 +1091,10 @@ IOUSBControllerV2::ReturnIsochDoneQueue(IOUSBControllerIsochEndpoint* pEP)
 			{
 				USBLog(1, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - kIOReturnOverrun on IN - device babbling?", this);
 			}
-			USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue- calling handler(target: %p, comp.param: %p, status: %p, pFrames: %p)", this,
-				   pTD->_completion.target, pTD->_completion.parameter, (void*)pEP->accumulatedStatus, pFrames);
+
+			USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue- TD (%p) calling handler[%p](target: %p, comp.param: %p, status: %p, pFrames: %p)", this, pTD,
+																pHandler, pTD->_completion.target, pTD->_completion.parameter, (void*)pEP->accumulatedStatus, pFrames);
+			
 			(*pHandler) (pTD->_completion.target,  pTD->_completion.parameter, pEP->accumulatedStatus, pFrames);
 			
 			_activeIsochTransfers--;
@@ -983,8 +1128,28 @@ IOUSBControllerV2::ReturnIsochDoneQueue(IOUSBControllerIsochEndpoint* pEP)
 
 
 
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  20);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  21);
+// This method should be overridden by any controller which can support >4GB physical addresses (32 bits addresses)
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  20);
+IODMACommand*
+IOUSBControllerV2::GetNewDMACommand()
+{
+	return IODMACommand::withSpecification(kIODMACommandOutputHost64, 32, 0);
+}
+
+
+
+#define defaultOptionBits						0							// by default we don't need contiguous memory
+#define defaultPhysicalMask						0x00000000FFFFF000ULL		// be default we require memory 4K aligned memory below 4GB
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  21);
+IOReturn
+IOUSBControllerV2::GetLowLatencyOptionsAndPhysicalMask(IOOptionBits *pOptionBits, mach_vm_address_t *pPhysicalMask)
+{
+	*pOptionBits = defaultOptionBits;
+	*pPhysicalMask = defaultPhysicalMask;
+	return kIOReturnSuccess;
+}
+
+
 OSMetaClassDefineReservedUnused(IOUSBControllerV2,  22);
 OSMetaClassDefineReservedUnused(IOUSBControllerV2,  23);
 OSMetaClassDefineReservedUnused(IOUSBControllerV2,  24);
