@@ -301,8 +301,7 @@ IOUSBPipe::Read(IOMemoryDescriptor * buffer, UInt64 frameStart, UInt32 numFrames
     }
     else
     {
-        err = _controller->IsocIO(buffer, frameStart, numFrames, pFrames,
-                _address, &_endpoint, completion);
+        err = _controller->IsocIO(buffer, frameStart, numFrames, pFrames, _address, &_endpoint, completion);
     }
 
     return(err);
@@ -452,7 +451,7 @@ IOUSBPipe::Read(IOMemoryDescriptor *buffer, UInt32 noDataTimeout, UInt32 complet
     if (!buffer)
     {
         USBLog(5, "IOUSBPipe[%p]::Read - NULL buffer!", this);
-	return kIOReturnBadArgument;
+		return kIOReturnBadArgument;
     }
     
     return Read(buffer, noDataTimeout, completionTimeout, buffer->getLength(), completion, bytesRead);
@@ -572,7 +571,7 @@ IOUSBPipe::Read(IOMemoryDescriptor *buffer, UInt32 noDataTimeout, UInt32 complet
     if ((_endpoint.transferType != kUSBBulk) && (noDataTimeout || completionTimeout))
     {
         USBLog(5, "IOUSBPipe[%p]::Read - bad arguments:  (EP type: %d != kUSBBulk(%d)) && ( dataTimeout: %ld || completionTimeout: %ld)", this, _endpoint.transferType, kUSBBulk, noDataTimeout, completionTimeout);
-	return kIOReturnBadArgument;
+		return kIOReturnBadArgument;
     }
 
     if (!buffer || (buffer->getLength() < reqCount))
@@ -645,7 +644,7 @@ IOUSBPipe::Write(IOMemoryDescriptor *buffer, UInt32 noDataTimeout, UInt32 comple
     if (!buffer || (buffer->getLength() < reqCount))
     {
         USBLog(5, "IOUSBPipe[%p]::Write - bad buffer: (buffer %p) || ( length %ld < reqCount %ld)", this, buffer, buffer->getLength(), reqCount);
-	return kIOReturnBadArgument;
+		return kIOReturnBadArgument;
     }
 
     if (_correctStatus == kIOUSBPipeStalled)
@@ -711,6 +710,8 @@ IOUSBPipe::GetPipeStatus(void)
     return _correctStatus;
 }
 
+
+
 OSMetaClassDefineReservedUsed(IOUSBPipe,  7);
 IOReturn 
 IOUSBPipe::ClearPipeStall(bool withDeviceRequest)
@@ -720,6 +721,7 @@ IOUSBPipe::ClearPipeStall(bool withDeviceRequest)
     USBLog(5,"IOUSBPipe[%p]::ClearPipeStall",this);
     if (_correctStatus != 0)
         USBLog(2, "IOUSBPipe[%p]::ClearPipeStall setting status to 0", this);
+
     _correctStatus = 0;
     
     if ( _endpoint.transferType == kUSBIsoc )
@@ -732,60 +734,60 @@ IOUSBPipe::ClearPipeStall(bool withDeviceRequest)
 
     if (_device->GetSpeed() == kUSBDeviceSpeedHigh)
     {
-	USBLog(5,"IOUSBPipe[%p]::ClearPipeStall - High Speed Device, no clear TT needed",this);
+		USBLog(5,"IOUSBPipe[%p]::ClearPipeStall - High Speed Device, no clear TT needed",this);
     }
     else if(!err && ( (_endpoint.transferType == kUSBBulk) || (_endpoint.transferType == kUSBControl) ) )
     {
-	USBLog(5,"IOUSBPipe[%p]::ClearPipeStall Bulk or Control endpoint, clear TT",this);
-	// Now, we need to tell our parent to issue a port reset to our port.  Our parent is an IOUSBDevice
-	// that has a hub driver attached to it.  However, we don't know what kind of driver that is, so we
-	// just send a message to all the clients of our parent.  The hub driver will be the only one that
-	// should do anything with that message.
-	//
-	if( (_device != NULL) && (_device->_expansionData->_usbPlaneParent ) )
-	{
-	    IOUSBHubPortClearTTParam params;
-	    UInt8 	 deviceAddress;  //<<0
-	    UInt8	 endpointNum;    //<<8
-	    UInt8 	 endpointType;	 //<<16 // As split transaction. 00 Control, 10 Bulk
-	    UInt8 	 in;		 //<<24 // Direction, 1 = IN, 0 = OUT};
-	
-	    params.portNumber = _device->_expansionData->_portNumber;
-	    deviceAddress = _device->GetAddress();
-	    endpointNum = _endpoint.number;
-	    if(_endpoint.transferType == kUSBControl)
-	    {
-		endpointType = 0;	// As split transaction. 00 Control, 10 Bulk
-		in = 0;			// Direction, 1 = IN, 0 = OUT, not used for control
-	    }
-	    else
-	    {
-		endpointType = 2;		// As split transaction. 00 Control, 10 Bulk
-		if(_endpoint.direction == kUSBIn)
+		USBLog(5,"IOUSBPipe[%p]::ClearPipeStall Bulk or Control endpoint, clear TT",this);
+		// Now, we need to tell our parent to issue a port reset to our port.  Our parent is an IOUSBDevice
+		// that has a hub driver attached to it.  However, we don't know what kind of driver that is, so we
+		// just send a message to all the clients of our parent.  The hub driver will be the only one that
+		// should do anything with that message.
+		//
+		if( (_device != NULL) && (_device->_expansionData->_usbPlaneParent ) )
 		{
-		    in = 1;			// Direction, 1 = IN, 0 = OUT, not used for control
+			IOUSBHubPortClearTTParam	params;
+			UInt8						deviceAddress;		//<<0
+			UInt8						endpointNum;		//<<8
+			UInt8						endpointType;		//<<16 // As split transaction. 00 Control, 10 Bulk
+			UInt8						in;					//<<24 // Direction, 1 = IN, 0 = OUT};
+		
+			params.portNumber = _device->_expansionData->_portNumber;
+			deviceAddress = _device->GetAddress();
+			endpointNum = _endpoint.number;
+			if(_endpoint.transferType == kUSBControl)
+			{
+				endpointType = 0;	// As split transaction. 00 Control, 10 Bulk
+				in = 0;				// Direction, 1 = IN, 0 = OUT, not used for control
+			}
+			else
+			{
+				endpointType = 2;		// As split transaction. 00 Control, 10 Bulk
+				if(_endpoint.direction == kUSBIn)
+				{
+					in = 1;			// Direction, 1 = IN, 0 = OUT, not used for control
+				}
+				else
+				{
+					in = 0;
+				}
+			}
+			params.options = deviceAddress + (endpointNum <<8) + (endpointType << 16) + (in << 24);
+			
+			USBLog(5, "[%p] ClearPipeStall, calling device messageClients (kIOUSBMessageHubPortClearTT) with options: 0x%lx", this, params.options);
+			_device->_expansionData->_usbPlaneParent->retain();
+			(void) _device->_expansionData->_usbPlaneParent->messageClients(kIOUSBMessageHubPortClearTT, &params, sizeof(params));
+			_device->_expansionData->_usbPlaneParent->release();
 		}
-		else
-		{
-		    in = 0;
-		}
-	    }
-	    params.options = deviceAddress + (endpointNum <<8) + (endpointType << 16) + (in << 24);
-	    
-	    USBLog(5, "[%p] ClearPipeStall, calling device messageClients (kIOUSBMessageHubPortClearTT) with options: 0x%lx", this, params.options);
-	    _device->_expansionData->_usbPlaneParent->retain();
-	    (void) _device->_expansionData->_usbPlaneParent->messageClients(kIOUSBMessageHubPortClearTT, &params, sizeof(params));
-	    _device->_expansionData->_usbPlaneParent->release();
-	}
     }
     else
     {
-	USBLog(5,"IOUSBPipe[%p]::ClearPipeStall Int or Isoc endpoint, don't clear TT (or err: 0x%x)",this, err);
+		USBLog(5,"IOUSBPipe[%p]::ClearPipeStall Int or Isoc endpoint, don't clear TT (or err: 0x%x)",this, err);
     }
     
     if (!err && withDeviceRequest)
     {
-	IOUSBDevRequest	request;
+		IOUSBDevRequest	request;
         IOUSBCompletion	tap;
 
 		// The action of IOUSBSyncCompletion will tell the USL that this is a sync transfer
@@ -795,15 +797,15 @@ IOUSBPipe::ClearPipeStall(bool withDeviceRequest)
         tap.parameter = &request.wLenDone;
 
         USBLog(2,"IOUSBPipe[%p]::ClearPipeStall - sending request to the device", this);
-	request.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBStandard, kUSBEndpoint);
-	request.bRequest = kUSBRqClearFeature;
-	request.wValue = kUSBFeatureEndpointStall;
-	request.wIndex = _endpoint.number | ((_endpoint.direction == kUSBIn) ? 0x80 : 0);
-	request.wLenDone = request.wLength = 0;
-	request.pData = NULL;
+		request.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBStandard, kUSBEndpoint);
+		request.bRequest = kUSBRqClearFeature;
+		request.wValue = kUSBFeatureEndpointStall;
+		request.wIndex = _endpoint.number | ((_endpoint.direction == kUSBIn) ? 0x80 : 0);
+		request.wLenDone = request.wLength = 0;
+		request.pData = NULL;
 
-	// send the request to pipe zero
-	err = _controller->DeviceRequest(&request, &tap, _address, 0, kUSBDefaultControlNoDataTimeoutMS, kUSBDefaultControlCompletionTimeoutMS);
+		// send the request to pipe zero
+		err = _controller->DeviceRequest(&request, &tap, _address, 0, kUSBDefaultControlNoDataTimeoutMS, kUSBDefaultControlCompletionTimeoutMS);
 
     }
     return err;
@@ -872,9 +874,9 @@ OSMetaClassDefineReservedUsed(IOUSBPipe,  9);
 // Isochronous read with frame list updated at hardware interrupt time
 //
 IOReturn 
-IOUSBPipe::Read( IOMemoryDescriptor *	buffer,
-                 UInt64 frameStart, UInt32 numFrames, IOUSBLowLatencyIsocFrame *pFrames,
-                          IOUSBLowLatencyIsocCompletion *	completion, UInt32 updateFrequency)
+IOUSBPipe::Read(IOMemoryDescriptor *	buffer,
+				UInt64 frameStart, UInt32 numFrames, IOUSBLowLatencyIsocFrame *pFrames,
+				IOUSBLowLatencyIsocCompletion *	completion, UInt32 updateFrequency)
 {
     IOReturn	err = kIOReturnSuccess;
 
