@@ -102,6 +102,7 @@ protected:
 		bool					_suspendCommand;
 		IOCommandGate *			_commandGate;
 		OSSet *					_openInterfaces;
+		bool					_resetCommand;
     };
     ExpansionData * _expansionData;
 
@@ -138,11 +139,13 @@ public:
     virtual bool 	willTerminate( IOService * provider, IOOptionBits options );
     virtual bool 	didTerminate( IOService * provider, IOOptionBits options, bool * defer );
 	
+#if !(defined(__ppc__) && defined(KPI_10_4_0_PPC_COMPAT))
 	virtual bool	handleIsOpen(const IOService *forClient) const;
 	virtual bool	handleOpen(IOService *forClient, IOOptionBits options, void *arg);
 	virtual void	handleClose(IOService *forClient, IOOptionBits options);
     virtual bool	terminate( IOOptionBits options = 0 );
     virtual bool	requestTerminate( IOService * provider, IOOptionBits options );
+#endif
 
     virtual void SetPort(void *port);			// Obsolete, do NOT use
 
@@ -393,6 +396,7 @@ public:
      */
     virtual void	DisplayUserNotification(UInt32 notificationType);
     
+#if !(defined(__ppc__) && defined(KPI_10_4_0_PPC_COMPAT))
     OSMetaClassDeclareReservedUsed(IOUSBDevice,  5);
 	/*!
         @function MakePipe
@@ -402,6 +406,9 @@ public:
 	 */
     virtual IOUSBPipe*	MakePipe(const IOUSBEndpointDescriptor *ep, IOUSBInterface *interface);
     
+#else
+    OSMetaClassDeclareReservedUnused(IOUSBDevice,  5);
+#endif
 	
     OSMetaClassDeclareReservedUnused(IOUSBDevice,  6);
     OSMetaClassDeclareReservedUnused(IOUSBDevice,  7);
