@@ -409,6 +409,7 @@ Completion Code         Error Returned              Description
 #define kIOUSBMessagePortHasBeenSuspended   iokit_usb_msg(13)  // 0xe0000400d  Message sent to a device indicating that the port it is attached to has been suspended
 #define kIOUSBMessageFromThirdParty         iokit_usb_msg(14)  // 0xe0000400e  Message sent from a third party.  Uses IOUSBThirdPartyParam to encode the sender's ID
 #define kIOUSBMessagePortWasNotSuspended    iokit_usb_msg(15)  // 0xe0000400f  Message indicating that the hub driver received a resume request for a port that was not suspended
+#define kIOUSBMessageExpressCardCantWake    iokit_usb_msg(16)  // 0xe00004010  Message from a driver to a bus that an express card will disconnect on sleep and thus shouldn't wake
 
 // Obsolete
 //
@@ -985,6 +986,7 @@ enum {
 #define kUSBProductIDMask						"idProductMask"
 #define kUSBPreferredConfiguration				"Preferred Configuration"
 #define kUSBSuspendPort							"kSuspendPort"
+#define kUSBExpressCardCantWake					"ExpressCardCantWake"
 #define kUSBControllerNeedsContiguousMemoryForIsoch	"Need contiguous memory for isoch"
 /*!
 @enum USBReEnumerateOptions
